use basic;
use cfgfile;
use os;
use uo;

include "NPC_Stablemaster";

var mrcspawn_cfg := ReadConfigFile( "::mrcspawn" );

const MAX_SKILLS      := 48;  
const UOBJ_GOLDCOIN   := 0x0EED;

function SendVendorGump(who, vendorFunctions)

	var textY := 128;
	var layout := array(
		"page 0", 
		"nodispose 1",
		"resizepic 0 50 5170 230 400",
		"gumppic 25 85 92",
		"gumppic 70 86 94",
		
		"text 80 91 48 0" //Vendor Control
	);

	var data := array(
		"Vendor Control"
	);
	
	foreach funct in vendorFunctions
		if (funct != "-")
			layout.append("button 30 " + textY + " 2440 2440 1 0 " + _funct_iter);
			layout.append("text 40 " + (textY+2) + " 1001 " + _funct_iter);
		endif
			data.append(funct);
		textY := textY + 20;
	endforeach
	layout[3] := "resizepic 0 50 5170 230 "+textY;

	var res := SendDialogGump(who,layout,data);
	return({ res, vendorFunctions[res[0]] } );
endfunction

function find_or_create_item(storage, name, objtype)
  var item := FindRootItemInStorageArea(storage, name);
  if(item)
	return item;
  endif
  item := CreateRootItemInStorageArea( storage, name, objtype );
  return item;
endfunction

function RestockInventory( merchanttype, byref inventory )
    foreach productgroup in ProductGroups(merchanttype)
        Restock( inventory, productgroup );
    endforeach
endfunction

function ProductGroups( mtype )
    var mtype_elem := FindConfigElem( mrcspawn_cfg, mtype );
    if (!mtype_elem)
        syslog( "Unable to find config elem " + mtype );
        return 0;
    endif
    return GetConfigStringArray( mtype_elem, "Sells" );
endfunction

function Restock( byref inventory, productgroup )
    var pg_elem := FindConfigElem( mrcspawn_cfg, productgroup );
    foreach itemstr in GetConfigStringArray( pg_elem, "Item" )
        sleepms(100);
        SpawnItem( inventory, itemstr );
    endforeach
endfunction

function SpawnItem( byref inventory, itemstr )
    var ii := SplitWords( itemstr );
    var objtype := Cint(ii[1]);
    var count   := CInt(ii[2]);
    count := count - CountExisting( inventory, objtype );
    if (count > 0)
      CreateItemInInventory( inventory, objtype, count);
    endif
    If (Count < 0)
      ConsumeSubstance( inventory, objtype, (count*-1) );
    Endif
endfunction

function CountExisting( byref container, objtype )
    var count := 0;
    foreach item in EnumerateItemsInContainer( container )
        if (item.objtype == objtype)
            count := count + item.amount;
        endif
    endforeach
    return count;
endfunction

function MerchantGroups()
    var mrcgroups_elem := FindConfigElem( mrcspawn_cfg, "MerchantGroups" );
    if (!mrcgroups_elem)
        syslog( "Unable to find 'MerchantGroups' element in mrcspawn.cfg" );
        return 0;
    endif
    return GetConfigStringArray( mrcgroups_elem, "MerchantGroup" );
endfunction

function SellAll(npc, who)
	PrintTextAbovePrivate( npc, "Which kind of item would you like to sell?", who );
	var tgt := Target(who);
	if (tgt)
		var item_cfg := ReadConfigFile("::itemdesc");
		var item_elm := FindConfigElem(item_cfg,tgt.objtype);
		var iSellPricePerPiece := GetConfigInt(item_elm,"VendorBuysFor");
		var TotalSell := 0;
		var iFound := 0;
		var iObjType := tgt.objtype;
		var iFull := 0;
		var reserved := 0;
		if (iSellPricePerPiece)
		    var items := EnumerateItemsInContainer(who.backpack);
		    foreach item in items
			  if(!ReserveItem(item))
				reserved:= 1;
			  else
				ReleaseItem(item);
			  endif
			  if (item.container.serial == who.backpack.serial)
				if (item.objtype == iObjType)
				    if ((Len(EnumerateItemsInContainer(item)) < 1) and (!item.newbie) and reserved != 1)
					ReleaseItem(item);
					iFound := 1;
					if (item.buyprice)
        				  if (item.amount)
        					TotalSell := TotalSell + (item.buyprice * item.amount);
        				  else
        					TotalSell := TotalSell + item.buyprice;
        				  endif
					else
        				  if (item.amount)
        					TotalSell := TotalSell + (iSellPricePerPiece * item.amount);
        				  else
        					TotalSell := TotalSell + iSellPricePerPiece;
        				  endif
        				endif
    					DestroyItem(item);
    				    endif
				endif
			  endif
		    endforeach
		    if (iFound)
			  PrintTextAbovePrivate( npc,"The total of thy sale is " + TotalSell,who);
			  if (TotalSell > 60000)
				var iLeft := TotalSell;
				while (iLeft > 0)
				    if (iLeft > 60000)
					if (!CreateItemInContainer(who.backpack,0xEED,60000))
					  CreateItemAtLocation(who.x,who.y,who.z,0xEED,60000);
					  iFull := 1;
					endif
					iLeft := iLeft - 60000;
				    else
					if (!CreateItemInContainer(who.backpack,0xEED,iLeft))
					  CreateItemAtLocation(who.x,who.y,who.z,0xEED,iLeft);
					  iFull := 1;
					endif
					iLeft := 0;
				    endif
				endwhile
			  else
				if (!CreateItemInContainer(who.backpack,0xEED,TotalSell))
				    CreateItemAtLocation(who.x,who.y,who.z,0xEED,TotalSell);
				    iFull := 1;
				endif
			  endif
			  if (iFull)
	    			SendSysMessage(who,"Your backpack is full, placing gold on ground");
	    		  endif
	    	    else
	    		PrintTextAbovePrivate( npc,"You don't have any of that item.",who);
		    endif
		else
		    PrintTextAbovePrivate( npc,"Bah, I have no use for this. Be off with ye.",who);
		endif
	else
		SendSysMessage(who,"Canceled");
	endif
endfunction

function SellBag(npc, who)
	PrintTextAbovePrivate( npc,"Which bag do you want to sell.",who);
	var tgt := Target(who);
	if (tgt.container.container.serial != who.serial)
		PrintTextAbovePrivate( npc,"That doust not belong to you.",who);
		return;
	endif
	var items := EnumerateItemsInContainer(tgt);
	if (len(items) < 1)
		PrintTextAbovePrivate( npc,"That is not a bag or it's an empty bag.",who);
		return;
	endif
	var iFound := 0;
	var item_cfg := ReadConfigFile("::itemdesc");
	var TotalSell := 0;
	var iFull := 0;
	var reserved := 0;
	//hmmm I wonder if not Sell Bag doesn't create to much lag.
	foreach item in items
		var item_elm := FindConfigElem(item_cfg,item.objtype);
		var iSellPricePerPiece := GetConfigInt(item_elm,"VendorBuysFor");
		if(!ReserveItem(item))
			reserved:= 1;
		else
			ReleaseItem(item);
		endif
		if ((iSellPricePerPiece) and (tgt.serial == item.container.serial)
		    and (Len(EnumerateItemsInContainer(item)) < 1) and (!item.newbie) and (reserved != 1))
		    ReleaseItem(item);
		    iFound := 1;
		    if (item.buyprice)
			if (item.amount)
		  	    TotalSell := TotalSell + (item.buyprice * item.amount);
			else
			    TotalSell := TotalSell + item.buyprice;
			endif
		    else
			if (item.amount)
			    TotalSell := TotalSell + (iSellPricePerPiece * item.amount);
			else
			    TotalSell := TotalSell + iSellPricePerPiece;
			endif
		    endif
		    DestroyItem(item);
		endif
	endforeach
	if (iFound)
		PrintTextAbovePrivate( npc,"The total of thy sale is " + TotalSell,who);
		if (TotalSell > 60000)
		    var iLeft := TotalSell;
		    while (iLeft > 0)
			if (iLeft > 60000)
			    if (!CreateItemInContainer(who.backpack,0xEED,60000))
				CreateItemAtLocation(who.x,who.y,who.z,0xEED,60000);
				iFull := 1;
			    endif
			    iLeft := iLeft - 60000;
			else
			    if (!CreateItemInContainer(who.backpack,0xEED,iLeft))
				CreateItemAtLocation(who.x,who.y,who.z,0xEED,iLeft);
				iFull := 1;
			    endif
			    iLeft := 0;
			endif
		    endwhile
		else
		    if (!CreateItemInContainer(who.backpack,0xEED,TotalSell))
			CreateItemAtLocation(who.x,who.y,who.z,0xEED,TotalSell);
		  	iFull := 1;
		    endif
		endif
		if (iFull)
   		    SendSysMessage(who,"Your backpack is full, placing gold on ground");
   		endif
   	else
   		PrintTextAbovePrivate( npc,"Your bag got nothing that interests me.",who);
	endif

endfunction

// Used to inform a player if and how much a NPC can train a player
function MerchantTrain( me , who )

    var skillcfg := ReadConfigFile( "::skills" ); // Link to the skills cfg file 'pol\config\skills.cfg'
    var npcs:= ReadConfigFile("::npcdesc");
    var elem;             // used to contain the skill structure for the skill in question.
    var elem2;
    var i;                // Counter for various operations
    var totaltrain := 0;  // How many skills the player can be trained in <Used as a flag>

    var merchanttype := npcs[me.npctemplate].Merchanttype;

    if(!merchanttype)
	merchanttype := "Mage";
    endif

        var trainable := array;
	var skillidtrain := array;
        // For each of the available skills
        for( i := 0 ; i <= MAX_SKILLS ; i := i+1 )
            // acquire the skills attributes.
            elem := FindConfigElem( skillcfg , i );
            // if the NPC has the skill
	    elem2:= FindConfigElem(npcs,merchanttype);
            if(GetConfigString(elem2,GetConfigString(elem,"Name")))
                // if the NPC's skill is greater then the PC's
                if (33 > GetAttribute(who , getskillattrib(i)))
                  // Add the skill to the string, and increment total train.
                  trainable.append(GetConfigString( elem , "Name" ));
		  skillidtrain.append(i);
                  totaltrain := totaltrain + 1;
                endif
            endif
        endfor

        // If there where skills that the PC can be trained in, launch gump.
        if (totaltrain > 0)

		var textY := 128;
		var layout := array(
			"page 0", 
			"nodispose 1",
			"resizepic 0 50 5170 230 358",
			"gumppic 25 85 92",
			"gumppic 70 86 94",
			"text 80 91 48 0", //Training menu

			"page 1"
		);

		var data := array(
			"Training menu"
		);

		var c := 0;
		var d;
		foreach skill in trainable
			layout.append("button 30 " + textY + " 2440 2440 1 0 " + _skill_iter);
			layout.append("text 40 " + (textY+2) + " 1001 " + _skill_iter);
			case(lower(skill))
				"evaluatingintelligence":	data.append("Eval. Intelligence");
				"magicresistance":		data.append("Magic Resistance");
				"detectinghidden":		data.append("Detecting Hidden");
				"animallore":			data.append("Animal Lore");
				"animaltaming":			data.append("Animal Taming");
				"itemid":			data.append("Item Identification");
				"spiritspeak":			data.append("Spirit Speak");
				"removetrap":			data.append("Remove Trap");
				"tasteidentification":		data.append("Taste Identification");
				"armslore":			data.append("Arms Lore");
				default:			data.append(skill);
			endcase
			textY := textY + 20;
			c := c+1;
			if(c==10)
				layout.append("button 175 " + (textY+20) + " 5601 5605 0 2 0");
				textY := 128;
				layout.append("page 2");
			endif
			if(c==20 || c==30 || c==40)
				d := c/10;
				layout.append("button 35 " + (textY+20) + " 5603 5607 0 "+(d-1)+" 0");
				layout.append("button 175 " + (textY+20) + " 5601 5605 0 "+(d+1)+" 0");
				textY := 128;
				layout.append("page "+(d+1));
			endif
		endforeach
		if(c>10)
			d := c/10;
			layout.append("button 35 348 5603 5607 0 "+d+" 0");
		elseif(c==10)
			layout.erase(28);
			layout[3] := "resizepic 0 50 5170 230 338";
		else
			layout[3] := "resizepic 0 50 5170 230 "+textY;
		endif

		var res := SendDialogGump(who,layout,data);
		if(res)
			elem := FindConfigElem( skillcfg , skillidtrain[res[0]] );
			var NPC_level := GetConfigInt( elem2, trainable[res[0]] );

			var PC_level := GetBaseSkill( who , skillidtrain[res[0]] );
			if(PC_level >= 33)
				PrintTextAbovePrivate( me, "I can not train thee.", who);
			else
				var trainmaxcost := GoldForSkillGain(CInt(NPC_level/3), PC_level);
				PrintTextAbovePrivate(me, "I will train thee " + GetConfigString( elem , "Name" ) + " for " + trainmaxcost + " gold.", who);
				sleep(1);
				PrintTextAbovePrivate(me, "Just give me the gold if you want the job done.", who);
				SetObjProperty( who , "TrainMaxCost" , trainmaxcost );
				SetObjProperty( who , "TrainSkillID" , skillidtrain[res[0]]);
				return;
			endif
		endif
	else
		PrintTextAbovePrivate( me, "I can not train thee.", who);
	endif
endfunction

// This handles the NPC's training of the PC or the PC's donation of gold to the NPC
// TODO: Reputation bonus if gold donation is greater then a specific amount.
function TrainSkill( me , who , item )

  var skillcfg := ReadConfigFile( "::skills" ); // Link to master skills config file
  var npcs:= ReadConfigFile("::npcdesc");
  var variance := 0;       // The variance between the players skill and the npc's skill
  var NPC_level := 0;      // The NPC's level at the skill
  var PC_level := 0;       // The PC's level at the skill
  var GoldReceived := 0;   // The amount of gold the PC did give us.
  var New_Level := 0;      // The PC's new skill level
  var CurrentNPCCost := 0; // The current NPC's cost for training the Player
  var elem;
  var elem2;

  // if the item handed to the NPC was gold and not zero <Integrity check>
  if( (item.objtype == UOBJ_GOLDCOIN) and (item.amount != 0)  )
     // Acquire information on what skill the PC whishes to train in.
     var TrainMaxCost := GetObjProperty( who , "TrainMaxCost" );
     var TrainSkillID := GetObjProperty( who , "TrainSkillID" );

     // If the pc is not expecting training, then it was a donation.
     if( (!TrainMaxCost or !TrainSkillID) and TrainSkillID != 0)
         PrintTextAbovePrivate(me, "I don't know what this is for, but thanks!", who );
         DestroyItem( item );
         // TODO: Reputation bonus
         // TODO: OSI has a "Desires" property with all npc's.  It's used to determine if an NPC gives out clues to a minor quest.
         return;
     endif

     // If the NPC does have the skill
     if( GetAttribute(me , getskillattrib(TrainSkillID)) )
          elem         := FindConfigElem(skillcfg,TrainSkillID);
	  elem2 := npcs[me.npctemplate].Merchanttype;

	if(!elem2)
	  elem2 := "Mage";
	endif

	  NPC_level    := GetConfigString(elem2,GetConfigString(elem,"Name"));
          PC_level     := GetAttribute(who, getskillattrib(TrainSkillID));     // Acquire PC's  level
          GoldReceived := item.amount;                     // How much gold was received
          variance     := 33 - PC_level;            // How many skill points are there between the npc and player.
          CurrentNPCCost := GoldForSkillGain(33, PC_level);

          // Double check to make sure that the pc's skill is lower then the NPC's
          if (PC_level < 33)

            // If what the NPC can train the player is different from what the player is expecting, adjust accordingly.
            if (CurrentNPCCost != TrainMaxCost)

              if (CurrentNPCCost > TrainMaxCost)
                // The current NPC is better then the one we received an offer from.
                NPC_Level := SkillGainForGold(NPC_level, PC_level, TrainSkillID, TrainMaxCost);  // Adjust the NPC's level variable to the lower skill level
                variance  := 33 - PC_level;
              else
                // The current NPC is worse then the one we received an offer from.
                // Determine what the cost would be for this NPC
                if (GoldReceived > CurrentNPCCost)
                  // The NPC can't help the player
                  PrintTextAbovePrivate(me, "I can not train thee to that level.", who);
                  MoveItemToContainer( who.backpack , item );
                  return;
                else
                  // The NPC can help, but we need to adjust
                  TrainMaxCost := CurrentNPCCost;
                endif
              endif
            endif

            // Determine what level the player will be trained to
            // if the PC gives the NPC the max cost or more. Train them to the fullest
            if( TrainMaxCost <= GoldReceived )
              New_Level := (33 * 10);
              EraseObjProperty( who , "TrainMaxCost" );  // Clear the trainmaxcost property on the PC.
              EraseObjProperty( who , "TrainSkillID" );  // Clear the skill flag
              // TODO: If the player gives a lot more then the training cost, grant a Reputation Bonus and/or a mini-quest clue.
            else
              // Train the PC to whatever percentage they can afford.
              New_Level := ((PC_level + CInt((CDbl(variance) * (CDbl(GoldReceived) / CDbl(TrainMaxCost))))) * 10);
              SetObjProperty( who , "TrainMaxCost" , TrainMaxCost-GoldReceived );
            endif

            PrintTextAbovePrivate(me, "Let me show you how it's done.", who);  // NPC informs player training was successful

            // Set the PC's skill
            SetAttributeBaseValue( who, getskillattrib(TrainSkillID), New_Level );

            SubtractAmount( item, TrainMaxCost );  // Destroy gold

            SendSysmessage( who , "Your skill increases." ); // Note increase

          else
            // PC just paid NPC to be trained but NPC doesn't have the skill.
            PrintTextAbovePrivate(me, "I'm not as good as thee.  Please find another to train thee.", who);
            MoveItemToContainer( who.backpack , item );
          endif
      else
          // NPC does not have that skill to train the PC.  Return gold
          PrintTextAbovePrivate(me, "I don't know that skill. Why don't you ask me what i can train you in?", who);
          MoveItemToContainer( who.backpack , item );
      endif
  else
     // PC gave NPC something else other then gold.  Return it.
     PrintTextAbovePrivate(me, "Bah, I have no use for this. Be off with ye.", who);
     MoveItemToContainer( who.backpack , item );
     // TODO: OSI has a "Desires" property with all npc's.  I think it's used to determine if an NPC
     //  gives out clues to a minor quest or reputation bonus, depending on the item donated <Type and value>
  endif

endfunction

// Used to determine cost for raising the pc's skill to the level of the NPC.
// Note: This function is designed to handle different skills have different rates at different levels, etc.
function GoldForSkillGain( NPC_Skill, PC_Skill )
  var maxskill := NPC_Skill - PC_Skill;
  return maxskill*10; // Cost equals ten times the difference.
endfunction

// Used to determine what the skill level of the PC will be for the amount of gold they gave
// Note: This function is designed to handle different skills have different rates at different levels, etc.
function SkillGainForGold(NPC_level, PC_level, TrainSkillID, GoldReceived)
  return (GoldReceived/10) + PC_Level;
endfunction
 
function getskillattrib( num )

case(num)
0: return ATTRIBUTEID_ALCHEMY;
1: return ATTRIBUTEID_ANATOMY;
2: return ATTRIBUTEID_ANIMALLORE;
3: return ATTRIBUTEID_ITEMID;
4: return ATTRIBUTEID_ARMSLORE;
5: return ATTRIBUTEID_PARRY;
6: return ATTRIBUTEID_BEGGING;
7: return ATTRIBUTEID_BLACKSMITHY;
8: return ATTRIBUTEID_BOWCRAFT;
9: return ATTRIBUTEID_PEACEMAKING;
10: return ATTRIBUTEID_CAMPING;
11: return ATTRIBUTEID_CARPENTRY;
12: return ATTRIBUTEID_CARTOGRAPHY;
13: return ATTRIBUTEID_COOKING;
14: return ATTRIBUTEID_DETECTINGHIDDEN;
15: return ATTRIBUTEID_ENTICEMENT;
16: return ATTRIBUTEID_EVALINT;
17: return ATTRIBUTEID_HEALING;
18: return ATTRIBUTEID_FISHING;
19: return ATTRIBUTEID_FORENSICS;
20: return ATTRIBUTEID_HERDING;
21: return ATTRIBUTEID_HIDING;
22: return ATTRIBUTEID_PROVOCATION;
23: return ATTRIBUTEID_INSCRIPTION;
24: return ATTRIBUTEID_LOCKPICKING;
25: return ATTRIBUTEID_MAGERY;
26: return ATTRIBUTEID_MAGICRESISTANCE;
27: return ATTRIBUTEID_TACTICS;
28: return ATTRIBUTEID_SNOOPING;          
29: return ATTRIBUTEID_MUSICIANSHIP;      
30: return ATTRIBUTEID_POISONING;        
31: return ATTRIBUTEID_ARCHERY;        
32: return ATTRIBUTEID_SPIRITSPEAK;       
33: return ATTRIBUTEID_STEALING;        
34: return ATTRIBUTEID_TAILORING; 
35: return ATTRIBUTEID_TAMING;            
36: return ATTRIBUTEID_TASTEID;          
37: return ATTRIBUTEID_TINKERING;      
38: return ATTRIBUTEID_TRACKING;         
39: return ATTRIBUTEID_VETERINARY;      
40: return ATTRIBUTEID_SWORDSMANSHIP;     
41: return ATTRIBUTEID_MACEFIGHTING;      
42: return ATTRIBUTEID_FENCING;          
43: return ATTRIBUTEID_WRESTLING;        
44: return ATTRIBUTEID_LUMBERJACKING;     
45: return ATTRIBUTEID_MINING;            
46: return ATTRIBUTEID_MEDITATION;        
47: return ATTRIBUTEID_STEALTH;           
48: return ATTRIBUTEID_REMOVETRAP;
default: return;        
endcase

endfunction

function Appraise(who, me)
  var trg := Target(who);
  if(!trg)
    PrintTextAbovePrivate(me, "Changed your mind?", who);
    return;
  endif
  var checker := 0;
  foreach thing in EnumerateItemsInContainer(who.backpack)
    if(thing == trg)
      checker := 1;
      break;
    endif
  endforeach
  if(checker == 0)
    PrintTextAbovePrivate(me, "But you dont own that!", who);
    return;
  endif
  checker := 0;
  var deedarray := { 0x6019, 0x601a, 0x601b, 0x601c, 0x601d, 0x601e, 0x6020, 0x6021, 0x6022, 0x6023,
		     0x6024, 0x6025, 0x6026, 0xd8a1, 0xd8a2, 0xd8a3, 0xd8a4, 0xd8a5, 0xd8a6, 0xd8a7,
		     0xd8a8, 0xd8a9, 0xd8aa, 0xd8ab, 0xd8ac, 0xd8ad, 0xd8ae, 0xd8af, 0xd8a0, 0xd8a1,
		     0xd8a2, 0xd8a3, 0xd8a4 };
  if(trg.objtype in deedarray)
    checker := 1;
  endif
  if(checker == 0)
    PrintTextAbovePrivate(me, "Im not interested in that!", who);
    return;
  endif
  var bankbox := FindBankBox(who);
  if(!bankbox)
    PrintTextAbove(me, "I cannot seem to find your bankbox!", who);
    return;
  endif
  var cnfg := ReadConfigFile(":housing:itemdesc");
  var amt := CInt(cnfg[trg.objtype].VendorSellsFor * 0.75);
  PrintTextAbovePrivate(me,"I will give you " + amt + " gold coins for that deed.", who);
endfunction

function ProcessDeedSale(who, me)
  var trg := Target(who);
  if(!trg)
    PrintTextAbovePrivate(me, "Changed your mind?", who);
    return;
  endif
  var checker := 0;
  var check := 0;
  var deedarray := { 0x6019, 0x601a, 0x601b, 0x601c, 0x601d, 0x601e, 0x6020, 0x6021, 0x6022, 0x6023,
		     0x6024, 0x6025, 0x6026, 0xd8a1, 0xd8a2, 0xd8a3, 0xd8a4, 0xd8a5, 0xd8a6, 0xd8a7,
		     0xd8a8, 0xd8a9, 0xd8aa, 0xd8ab, 0xd8ac, 0xd8ad, 0xd8ae, 0xd8af, 0xd8a0, 0xd8a1,
		     0xd8a2, 0xd8a3, 0xd8a4 };
  if(trg.objtype in deedarray)
    checker := 1;
  endif
  if(checker == 0)
    PrintTextAbovePrivate(me, "Im not interested in that!", who);
    return;
  endif
  foreach thing in EnumerateItemsInContainer(who.backpack)
    if(thing == trg)
      check := 1;
      break;
    endif
  endforeach
  if(check == 0)
    PrintTextAbovePrivate(me, "But you dont own that!", who);
    return;
  endif

  var bankbox := FindBankBox(who);
  if(!bankbox)
    PrintTextAbovePrivate(me, "I cannot seem to find your bankbox!", who);
    MoveItemToContainer(trg, who.backpack);
    return;
  endif
  var cnfg := ReadConfigFile(":housing:itemdesc");
  var amt := CInt(cnfg[trg.objtype].VendorSellsFor * 0.75);
  var holder;
  var goldarray := {};
  var chk := 0;
  while(amt > 0)
     holder := 0;
     if(amt > 60000)
       holder := CreateItemInContainer(bankbox, 0xeed, 60000);
       if(holder)
         goldarray.append(holder);
         amt := amt - 60000;
       else
         foreach thing in goldarray
           DestroyItem(thing);
         endforeach
         chk := 1;
         PrintTextAbovePrivate(me, "Your bank box is too full!", who);
         break;
      endif
    else
       holder := CreateItemInContainer(bankbox, 0xeed, amt);
       if(holder)
         if(!DestroyItem(trg))
           foreach thing in goldarray
             DestroyItem(thing);
           endforeach
           chk := 1;
           PrintTextAbovePrivate(me, "Your bank box is too full!", who);
         endif
         break;
       else
         foreach thing in goldarray
           DestroyItem(thing);
         endforeach
         chk := 1;
         PrintTextAbovePrivate(me, "Your bank box is too full!", who);
         break;
       endif
    endif

  endwhile
  if(chk == 0)
    PrintTextAbovePrivate(me, "The funds have been deposited in your bank box.", who);
  endif
endfunction

function LookieHere(who)
  var mobiles := ListMobilesNearLocation(who.x, who.y, who.z, 15);
  var guard, num, i, hogcall, guardzone, element, range, timer, elem;
  var cfgfile := ReadConfigFile("::gzone");
  var entries := GetConfigStringKeys(cfgfile);
  foreach listing in entries
    element :=  cfgfile[listing];
    range := element.range;
    range := SplitWords(range);
    if((who.x >= CInt(range[1])) && (who.x <= CInt(range[3])) && (who.y >= CInt(range[2])) && (who.y <= CInt(range[4])))
      guardzone := 1;
      break;
    endif
  endforeach
  foreach npc in mobiles
	if(!npc.acctname)
	  elem := FindConfigElem(npccfg, npc.npctemplate);
	  if(!elem.guardignore && !GetObjProperty(npc, "called"));
		if(npc.script != "tamed")
          case(RandomInt(4) + 1)
            1: hogcall := "Guards! They're killing people here!";
            2: hogcall := "Guards! I pay my taxes and no guards? Guards!";
            3: hogcall := "Help! They're dying here! Guards!";
            4: hogcall := "Guards! Come save us quick!";
            5: hogcall := "Ah! Help us! Guards!";
          endcase
          PrintTextAbove(me, hogcall);
          if(guardzone == 1)
		    SetObjProperty(npc, "called", 1);
            num := RandomInt(4) + 1;
            var count := 0;
            foreach mobile in ListMobilesNearLocation(me.x, me.y, me.z, 25)
              if(count == 3)
                break;
              else
		        if(mobile.npctemplate == "townguard")
		          var ev := array;
                  ev.+ type;
                  ev.+ source;
                  ev.type := EVID_PEACEMADE;
                  ev.source := npc;
                  SendEvent(mobile, ev);
                  ev.type := SYSEVENT_ENGAGED;
                  SendEvent(mobile, ev);
                  count := count + 1;
                endif
              endif
            endforeach
          endif
		endif
	  endif
    else
	  timer := GetObjProperty(npc, "guardstimer");
	  if(npc.criminal && (timer < ReadGameClock() || timer.errortext))
        foreach mobile in ListMobilesNearLocation(me.x, me.y, me.z, 15)
		  if(mobile.npctemplate == "townguard")
		    var ev := array;
            ev.+ type;
            ev.+ source;
            ev.type := EVID_PEACEMADE;
            ev.source := npc;
            SendEvent(mobile, ev);
            ev.type := SYSEVENT_ENGAGED;
            SendEvent(mobile, ev);
            SetObjProperty(npc, "guardkill", 1);
          endif
        endforeach
        SetObjProperty(npc, "guardstimer", ReadGameClock()+60);
	  endif
	endif
  endforeach
  sleep(1);
endfunction

function CoordDistance(x1, y1 ,x2 ,y2)
	var x_dist := abs(x1 - x2);
	var y_dist := abs(y1 - y2);
	
	if ( x_dist > y_dist)
		return x_dist;
	else
		return y_dist;
	endif
endfunction

function SendPatrolGump(who, npc)

	var textY := 168;
	var layout := array(
		"page 0", 
		"nodispose 1",
		"resizepic 0 50 5170 230 400",
		"gumppic 25 85 92",
		"gumppic 70 86 94",
		
		"text 80 91 48 0", //Patrol menu
		"button 30 128 2440 2440 1 0 1",
		"text 40 130 1001 1",//Add Patrol Location
		"button 30 148 2440 2440 1 0 2",
		"text 40 150 1001 2"//Reset NPC Patrol
	);

	var data := array(
		"Patrol Menu",
		"Add Patrol Location",
		"Reset NPC Patrol"
	);
	
	var patrolx := GetObjProperty(npc, "patrolx");
	var patroly := GetObjProperty(npc, "patroly");
	var patrolz := GetObjProperty(npc, "patrolz");
	var locy;
	var locz;
	if(!patrolx)
		patrolx := array;
		patroly := array;
		patrolz := array;
	endif
	foreach locx in patrolx
			layout.append("button 30 " + textY + " 2440 2440 1 0 " + (_locx_iter+2));
			layout.append("text 40 " + (textY+2) + " 1001 " + (_locx_iter+2));
			locy := patroly[_locx_iter];
			locz := patrolz[_locx_iter];
			
			data.append(""+locx+", "+locy+", "+locz);
			textY := textY + 20;
	endforeach
	layout[3] := "resizepic 0 50 5170 230 "+textY;

	var res := SendDialogGump(who,layout,data);
	if(!res)
		return;
	endif
	case(res[0])
		0:	 return;
		1:	 var tgt := TargetCoordinates(who);
			 if(!tgt)
				SendPatrolGump(who, npc);
				return;
			 endif
			 patrolx.append(tgt.x);
			 patroly.append(tgt.y);
			 patrolz.append(tgt.z);
		2:	 EraseObjProperty(npc, "walkto");
			 EraseObjProperty(npc, "reverse");
			 SendSysMessage(who, "NPC patroling resetted");
		default: patrolx.erase(res[0]-2);
			 patroly.erase(res[0]-2);
			 patrolz.erase(res[0]-2);
	endcase
	if(patrolx != {})
		SetObjProperty(npc, "patrolx", patrolx);
		SetObjProperty(npc, "patroly", patroly);
		SetObjProperty(npc, "patrolz", patrolz);
	else
		EraseObjProperty(npc, "patrolx");
		EraseObjProperty(npc, "patroly");
		EraseObjProperty(npc, "patrolz");
	endif

	SendPatrolGump(who, npc);
endfunction

function WalktoLocation(npc, loc1, loc2, loc3, nerve_name)
	var repeatSteps := 0;
	var distLastStep;
	var lastPos     := {};
	lastPos := {npc.x, npc.y};
	
	var check := 0;
	while ( CoordDistance(npc.x, npc.y, loc1, loc2))
		if ( npc.warmode )
			AI_EndNerve(npc, nerve_name);
			return;
		elseif(!check)
			AI_Move_XYZ(npc, loc1, loc2, NEMOVE_TOWARD, NEMOVE_WALK, NOWAKE, CoordDistance(npc.x, npc.y, loc1, loc2)*1.2);
			check := 1;
		endif
		sleepms(600);
		while ( Events_Waiting() )
			wait_for_event(120);
		endwhile
		distLastStep := CoordDistance(npc.x, npc.y, lastPos[1], lastPos[2]);
		if (distLastStep > 2)
			lastPos[1] := npc.x;
			lastPos[2] := npc.y;
			repeatSteps:= 0;
		else
			repeatSteps := repeatSteps + 1;
		endif
		if(CoordDistance(npc.x, npc.y, loc1, loc2) == 0)
			check := 0;
			break;
		endif
		if (repeatSteps > 6)
			MoveCharacterToLocation(npc, loc1, loc2, loc3, MOVECHAR_FORCELOCATION);
			check := 0;
			break;
		endif
	endwhile
endfunction

function NpcAnchor(npc, anchorLoc, anchorFacing, nerve_name);
	if ((anchorLoc != error) && (npc.warmode == 0) && RandomInt(10) == 1)
	   WalktoLocation(npc, anchorLoc[1], anchorLoc[2], anchorLoc[3], nerve_name);
	   if (anchorFacing != error)
		AI_Turn_XYZ(npc, anchorFacing[1], anchorFacing[2], NOWAKE);
	   endif
	endif
endfunction

function NpcPatrol(npc, patrolLocx, patrolLocy, patrolLocz, nerve_name)
	var walkto := GetObjProperty(npc, "walkto");
	if(!walkto)
		walkto := 1;
	endif
	var totalpatrol := patrolLocx.size();
	if ((patrolLocx != error) && (npc.warmode == 0))
	   WalktoLocation(npc, patrolLocx[walkto], patrolLocy[walkto], patrolLocz[walkto], nerve_name);
	   if(walkto == totalpatrol)
		SetObjProperty(npc, "reverse", 1);
	   elseif(walkto == 1)
		EraseObjProperty(npc, "reverse");
	   endif
	   if(GetObjProperty(npc, "reverse"))
		walkto := walkto-1;
	   else
		walkto := walkto+1;
	   endif
	   SetObjProperty(npc, "walkto", walkto);
	endif
endfunction

function FriendsSpeech(npc)
		var nexthello := GetObjProperty(npc,"nexthello");
		if (!nexthello)
			nexthello := ReadGameClock();
		elseif ( nexthello > ReadGameClock() )
			return;
		endif
		SetObjProperty(npc,"nexthello",ReadGameClock()+600);
		var player;
		foreach char in ListMobilesNearLocation( npc.x, npc.y, npc.z, 5 )
			if(char.acctname && !char.cmdlevel)
				player := char;
				break;
			endif
		endforeach
		if(!player)
			return;
		endif
		var myfriends := GetObjProperty(npc,"friends");
		if(!myfriends)
			myfriends := {};
		endif
		var i := 0;
		for( i := 1 ; i <= 3 ; i := i+1 )
			if (!myfriends[i])
				break;
			elseif (myfriends[i] == player.name)
				GreetFriend(npc, player.name, i);
				return;
			endif
		endfor
		//the speaker is not a friend yet
		var maxfriend := i - 1;
		if (i > 3)
			//too many friends, pick a slot at random
			i := RandomInt(3)+1;
			maxfriend := 3;
		endif
		//check loyalty to the friend in that friendslot
		var loyalty := GetObjProperty(npc,"loyalty");
		if(!loyalty)
			loyalty := {};
		endif
		if (!loyalty[i])
			loyalty[i]:=0;
		endif
		if ( myfriends[i] && (RandomInt(20) <= (loyalty[i]+10)) )
			//don't ditch the old friend
			i := RandomInt(maxfriend)+1;
			ChatAboutFriends(npc, i);
			return;
		endif
		//make a new friend
		if(RandomInt(5) < 4)
			PrintTextAbove(npc, "Say, " + player.name + ", You sure do look familiar...");
			myfriends[i] := player.name;
			loyalty[i] := 0;
			SetObjProperty(npc,"friends",myfriends);
			SetObjProperty(npc,"loyalty",loyalty);
		else
			i := RandomInt(maxfriend)+1;
			ChatAboutFriends(npc, i);
			return;
		endif
endfunction

function GreetFriend(me, friend, friendnum)

	var loy := GetObjProperty(me,"loyalty");
	var loyalty := loy[friendnum];
	if (!loyalty)
		loyalty := 0;
	endif
	if (loyalty >= 3)
		friend := "My "+ GetFriendTitle(loyalty) + friend;
	endif
	if ( RandomInt(3) == 1 )
		loyalty := loyalty + 1;
		if (loyalty > 9)
			loyalty := 9;
		endif
	endif
		loy[friendnum] := loyalty;
		SetObjProperty(me,"loyalty",loy);

	case (randomint(22))
		0:	PrintTextAbove(me,"Oh, " + friend + ", how's life?");
		1:	PrintTextAbove(me,"Surely, " + friend + ", I can be of great assistance.");
		2:	PrintTextAbove(me,"Glad you're back again, " + friend + ".");
		3:	PrintTextAbove(me,""+friend + ", tell me about your latest adventures.");
		4:	PrintTextAbove(me,"Oh, " + friend + ", you're getting to be a real regular here, eh?");
		5:	PrintTextAbove(me,"Well, " + friend + ", I'm glad you came back.");
		6:	PrintTextAbove(me,"Always good to see you, " + friend);
		7:	PrintTextAbove(me,""+friend + ", what I can do to make your travels easier?");
		8:	PrintTextAbove(me,"Nice to see you again, " + friend );
		9:	PrintTextAbove(me,"Good to see you, " + friend);
		10:	PrintTextAbove(me, friend + ", I'm glad you came to see me.");
		11:	PrintTextAbove(me, friend + ", it's always good to see you!");
		12:	PrintTextAbove(me, friend + ", I hope you are well.");
		13:	PrintTextAbove(me, "May life treat you right, " + friend);
		14:	PrintTextAbove(me,"Say, " + friend + " nice weather lately, isn't it?" );
		15:	PrintTextAbove(me, friend + ", sometimes I wonder about you.");
		16:	PrintTextAbove(me,""+friend + ", it's always a pleasure to serve you");
		17:	PrintTextAbove(me,""+friend + ", you are always welcome.");
		18:	PrintTextAbove(me,"Well aren't you something, " + friend + "!");
		19:	PrintTextAbove(me,""+friend + ", your presence brightens my day");
		20:	PrintTextAbove(me,""+friend + " it's far too long between visits!");
		21:	PrintTextAbove(me,"So, " + friend + " care to stay and chat a while?" );
	endcase
endfunction

function ChatAboutFriends(me, friendnum)
	var loy := GetObjProperty(me,"loyalty");
	var loyalty := loy[friendnum];
	if (!loyalty)
		loyalty := 0;
	endif
	var fri := GetObjProperty(me,"friends");
	var friend := fri[friendnum];
	if (!friend )
		return;
	endif
	if (loyalty >= 3)
		friend := "My "+ GetFriendTitle(loyalty) + friend;
	endif

	case (randomint(19))
		0:	PrintTextAbove(me,""+friend + " comes in here a lot.");
		1:	PrintTextAbove(me,""+friend + " is quite a tradesperson.");
		2:	PrintTextAbove(me,"I don't see enough of " + friend + ".");
		3:	PrintTextAbove(me,""+friend + " has been about town for a while now.");
		4:	PrintTextAbove(me,"Have you met " + friend + "?");
		5:	PrintTextAbove(me,"I've been swapping tales with " + friend + ".");
		6:	PrintTextAbove(me,""+friend + " seems to be doing well.");
		7:	PrintTextAbove(me,"I'm hoping to see " + friend + " soon.");
		8:	PrintTextAbove(me,"You should get together with " + friend + " and me at the pub sometime.");
		9:	PrintTextAbove(me,"You remind me of " + friend);
		10:	PrintTextAbove(me,""+friend + " is one of my best customers.");
		11:	PrintTextAbove(me,"Do you know " + friend + "?  Nice person!");
		12:	PrintTextAbove(me,"Did you hear " + friend + "'s death cry?  What a tragedy!");
		13:	PrintTextAbove(me,""+friend + "'s achievements are becoming legendary.");
		14:	PrintTextAbove(me,"I'm proud to have a friend like " + friend );
		15:	PrintTextAbove(me,""+friend + "'s antics are scripts for the bards.");
		16:	PrintTextAbove(me,""+friend + " sure is nice.");
		17:	PrintTextAbove(me,"I believe " + friend + " may be stepping out with my neighbor.");
		18:	PrintTextAbove(me,"You should look " + friend + " up.");
	endcase

endfunction

function GetFriendTitle(loyalty)

	case (loyalty)
	3:
	4:
		return "Friend, ";
	5: 
		return "Good Friend, ";
	6: 
		return "Great Friend, ";
	7: 
		return "Old Friend, ";
	8: 
		return "Dear Friend, ";
	9: 
		return "Dearest Friend, ";
	default: 
		return "";
	endcase

endfunction

function SayNews(npc)
  var news := GetGlobalProperty("news");
  if(!news)
    PrintTextAbove(npc, "No news is good news.");
    return;
  endif
  var i;
//  for(i := 1; i <= len(news); i := i + 1)
//    say(news[i]);
//    sleep(2);
//  endfor
  if ((ReadGameClock() - 10) > getobjproperty(npc,"lastnews"))
	PrintTextAbove(npc, ""+news); 
	SetObjProperty(npc,"lastnews",ReadGameClock());
  endif
endfunction