///////////////////////////////////////
// NPCs spells functions. /////////////
///////////////////////////////////////

use uo;
use os;
use util;
use basic;
use cfgfile;

include "include/spellRestrictions";
include "include/attributes";
include "include/client";

/////////////////////////
// Clumsy ///////////////
/////////////////////////
function cast_clumsy( caster, cast_on )
  var circle := 1;
  SetObjProperty(cast_on, "LastHit", {caster.name,caster.serial, "clumsy" });
  if(Reflected(cast_on))
	cast_on := caster;
  endif
  PlaySoundEffect(cast_on, SFX_SPELL_CURE);
  PlayObjectCenteredEffect( cast_on, FX_CURSE_EFFECT, 7,7);
  if(!CanMod(cast_on, "dex"))
	return;
  endif
  var magery := GetEffectiveSkill(caster, SKILLID_MAGERY);
  var mod_amount := Resisted(circle,caster,cast_on,(GetModAmount( magery )));
  var duration := GetModDuration( magery );
  DoTempMod(cast_on, "dex", 0 - mod_amount, duration);
endfunction

/////////////////////////
// Feeblemind ///////////
/////////////////////////
function cast_feeblemind( caster, cast_on )
  var circle := 1;
  SetObjProperty(cast_on, "LastHit", {caster.name,caster.serial, "feeblemind" });
  if (Reflected(cast_on))
	cast_on := caster;
  endif
  PlaySoundEffect(cast_on, SFX_SPELL_FEEBLEMIND);
  PlayObjectCenteredEffect( cast_on, FX_CURSE_EFFECT, 7, 7 );
  if (!CanMod(cast_on, "int"))
	return;
  endif
  var magery := GetEffectiveSkill(caster, SKILLID_MAGERY);
  var mod_amount := Resisted(circle,caster,cast_on,(GetModAmount( magery )));
  var duration := GetModDuration( magery );
  DoTempMod(cast_on, "int", 0 - mod_amount, duration);
endfunction

/////////////////////////
// Heal  ////////////////
/////////////////////////
function cast_heal( caster, cast_on )
  var healed := CInt(GetEffectiveSkill(caster, SKILLID_MAGERY) / 20) + 5;
  if(healed > 20)
	healed := 20;
  endif
  if(cast_on.isA(POLCLASS_NPC))
    healed := healed * 2;
  endif
  var evalint := Cint(GetEffectiveSkill(caster, SKILLID_EVALINT) / 10);
  healed := healed + evalint;
  HealDamage(cast_on, healed);
  PlayObjectCenteredEffect(cast_on, FX_SPARK_EFFECT, 7, 0x10);
  PlaySoundEffect(cast_on, SFX_SPELL_HEAL);
endfunction

/////////////////////////
// Magic Arrow///////////
/////////////////////////
function cast_magicarrow( caster, cast_on )
  var circle := 1;
  PlayMovingEffect( caster, cast_on, FX_MAGIC_ARROW, 5, 1 );
  PlaySoundEffect( caster, SFX_SPELL_MAGIC_ARROW );
  var dmg := RandomDiceRoll("1d8");
  if(Reflected(cast_on))
    PlayMovingEffect( cast_on, caster, FX_MAGIC_ARROW, 5, 1 );
    PlaySoundEffect( cast_on, SFX_SPELL_MAGIC_ARROW );
    ApplyRawDamage( caster, Resisted(circle,caster,cast_on,dmg) );
  else
    if(GetHp(cast_on) > 0)
	  SetObjProperty(cast_on, "LastHit", {caster.name,caster.serial, "magic arrow" });
      if(cast_on.isA(POLCLASS_NPC))
        dmg := dmg * 2;
      endif
      var evalint:= CInt(GetEffectiveSkill(caster,SKILLID_EVALINT));
      evalint:= CInt((evalint / 10) - RandomInt(evalint / 20));
      dmg:= dmg + evalint;
      ApplyRawDamage(cast_on, Resisted(circle,caster,cast_on,dmg));
    endif
  endif
endfunction

/////////////////////////
// Reactive Armor ///////
/////////////////////////
function cast_reactivearmor( caster, cast_on )
  PlaySoundEffect(cast_on, SFX_SPELL_REACTIVE_ARMOR);
  PlayObjectCenteredEffect( cast_on, FX_SPARK_EFFECT, 7, 0x10 );
  if(GetObjProperty(cast_on, "ar"))
	return;
  endif
  var magery := GetEffectiveSkill(caster, SKILLID_MAGERY);
  var mod_amount := magery / 10;
  var duration := CInt(RandomInt(10) + (magery / 2));
  SetObjProperty(cast_on, "ReactiveArmor", mod_amount);
  Detach();
  sleep(duration);
  if(GetObjProperty(cast_on, "ReactiveArmor"))
    PlaySoundEffect(cast_on, 0x1eb);
    EraseObjProperty(cast_on,"ReactiveArmor");
  endif
endfunction

/////////////////////////
// Weaken ///////////////
/////////////////////////
function cast_weaken( caster, cast_on )
  var circle := 1;
  SetObjProperty(cast_on, "LastHit", {caster.name,caster.serial, "weaken" });
  if(Reflected(cast_on))
	cast_on := caster;
  endif
  PlaySoundEffect(cast_on, SFX_SPELL_WEAKEN);
  PlayObjectCenteredEffect( cast_on, FX_CURSE_EFFECT, 7,7);
  if(!CanMod(cast_on, "str"))
	return;
  endif
  var magery := GetEffectiveSkill(caster, SKILLID_MAGERY);
  var mod_amount := Resisted(circle,caster,cast_on,(GetModAmount( magery )));
  var duration := GetModDuration( magery );
  DoTempMod(cast_on, "str", 0 - mod_amount, duration);
endfunction

/////////////////////////
// Agility //////////////
/////////////////////////
function cast_agility( caster, cast_on )
  PlaySoundEffect(caster, 0x1e8);
  PlayObjectCenteredEffect(caster, 0x373a, 7,7);
  if(!CanMod(cast_on, "dex"))
    return;
  endif
  var magery := GetEffectiveSkill(caster, SKILLID_MAGERY);
  var mod_amount := GetModAmount( magery );
  var duration := GetModDuration( magery );
  DoTempMod(cast_on, "dex", mod_amount, duration);
endfunction

/////////////////////////
// Cunning //////////////
/////////////////////////
function cast_cunning( caster, cast_on )
  PlaySoundEffect(cast_on, SFX_SPELL_CUNNING);
  PlayObjectCenteredEffect( cast_on, FX_BLESS_EFFECT, 7,7);
  if (!CanMod(cast_on, "int"))
	return;
  endif
  var magery := GetEffectiveSkill(caster, SKILLID_MAGERY);
  var mod_amount := GetModAmount( magery );
  var duration := GetModDuration( magery );
  DoTempMod(cast_on, "int", mod_amount, duration);
endfunction

/////////////////////////
// Cure /////////////////
/////////////////////////
function cast_cure( caster, cast_on )
  PlaySoundEffect(caster, SFX_SPELL_CURE);
  PlayObjectCenteredEffect( cast_on, FX_BLESS_EFFECT, 10,10);
  var poison_descr := GetObjProperty( cast_on,"poison_level" );
  var potency := Cint((GetMaxHP(cast_on) * CInt(GetEffectiveSkill(caster, SKILLID_MAGERY) / 5)) / 100.0);
  var poison_level := CInt(poison_descr);
  if(cast_on.isA(POLCLASS_NPC))
    var element := npccfg_sr[cast_on.npctemplate].elemental;
    if(element == "poison")
      var ev := array;
      ev.+ type;
      ev.+ source;
      ev.type := EVID_PEACEMADE;
      SendEvent(cast_on, ev);
      ev.source := caster;
      SendEvent(cast_on, ev);
      ev.type := EVID_ENGAGED;
      SendEvent(cast_on, ev);
      ApplyRawDamage(cast_on, potency);
    else
      if (poison_level >= 1)
        poison_level := poison_level - potency;
        if (poison_level >= 1)
	      SendSysMessage(cast_on, caster.name + " has partially cured you!");
        else
	      SendSysMessage(cast_on, caster.name + " has cured you completely!");
        endif
	  SetObjProperty(cast_on, "poison_level", CStr(poison_level));
      endif
    endif
  else
    if (poison_level >= 1)
      poison_level := poison_level - potency;
      if (poison_level >= 1)
	    SendSysMessage(cast_on, caster.name + " has partially cured you!");
      else
	    SendSysMessage(cast_on, caster.name + " has cured you completely!");
      endif
	  SetObjProperty(cast_on, "poison_level", CStr(poison_level));
    endif
  endif
endfunction

/////////////////////////
// Harm /////////////////
/////////////////////////
function cast_harm( caster, cast_on )
  var circle := 2;
  var dmg := RandomDiceRoll("1d12");
  SetObjProperty(cast_on, "LastHit", {caster.name,caster.serial, "harm" });
  if (Reflected(cast_on))
	cast_on := caster;
  endif
  PlaySoundEffect(cast_on, SFX_SPELL_HARM);
  PlayObjectCenteredEffect( cast_on, FX_GLOW_SPIKE, 0x0a,0x1e);
  if(GetHp(cast_on) > 0)
    if(cast_on.npctemplate)
      dmg := dmg * 2;
    endif
    var evalint:= CInt(GetEffectiveSkill(caster,SKILLID_EVALINT));
    evalint:= CInt((evalint / 8) - RandomInt(evalint / 16));
    dmg:= dmg + evalint;
    ApplyRawDamage(cast_on, Resisted(circle,caster,cast_on,dmg));
  endif
endfunction

/////////////////////////
// Protection ///////////
/////////////////////////
function cast_protection( caster, cast_on )
  PlaySoundEffect(cast_on, SFX_SPELL_PROTECTION);
  PlayObjectCenteredEffect(cast_on, FX_BLESS_EFFECT, 7,7);
  if (!CanMod(cast_on, "ar"))
	return;
  endif
  var magery := GetEffectiveSkill(caster, SKILLID_MAGERY);
  var mod_amount := (GetModAmount(magery) * 2);
  var duration := GetModDuration(magery);
  DoTempMod(cast_on, "ar", mod_amount, duration);
endfunction

/////////////////////////
// Strength /////////////
/////////////////////////
function cast_strength( caster, cast_on )
  PlaySoundEffect(cast_on, SFX_SPELL_STRENGTH);
  PlayObjectCenteredEffect( cast_on, FX_BLESS_EFFECT, 7,7);
  if (!CanMod(cast_on, "str"))
	return;
  endif
  var magery := GetEffectiveSkill(caster, SKILLID_MAGERY);
  var mod_amount := GetModAmount( magery );
  var duration := GetModDuration( magery );
  DoTempMod(cast_on, "str", mod_amount, duration);
endfunction

/////////////////////////
// Bless ////////////////
/////////////////////////
function cast_bless( caster, cast_on )
  PlaySoundEffect(cast_on, SFX_SPELL_BLESS);
  PlayObjectCenteredEffect( cast_on, FX_BLESS_EFFECT, 10,10);
  var magery := GetEffectiveSkill(caster, SKILLID_MAGERY);
  var evalint := GetEffectiveSkill(caster, SKILLID_EVALINT);
  var mod_amount := (magery / 10) + RandomInt(evalint / 10);
  var duration := GetModDuration(magery);
  if(CanMod(cast_on, "str"))
	DoTempMod(cast_on, "str", mod_amount, duration);
  endif
  if(CanMod(cast_on, "dex"))
	DoTempMod(cast_on, "dex", mod_amount, duration);
  endif
  if(CanMod(cast_on, "int"))
	DoTempMod(cast_on, "int", mod_amount, duration);
  endif
endfunction

/////////////////////////
// Fireball /////////////
/////////////////////////
function cast_fireball( caster, cast_on )
  var circle := 3;
  PlayMovingEffect(caster, cast_on, FX_FIREBALL, 5, 1);
  PlaySoundEffect(caster, 0x15f);
  var dmg := RandomDiceRoll("3d6");
  SetObjProperty(cast_on, "LastHit", {caster.name, caster.serial, "fireball"});
  if(Reflected(cast_on))
    PlayMovingEffect(cast_on, caster, FX_FIREBALL, 5, 1);
    PlaySoundEffect(cast_on, 0x15f);
    cast_on := caster;
  endif
  if(GetHp(cast_on) > 0)
    if(cast_on.npctemplate)
      dmg := dmg * 2;
    endif
    var evalint:= CInt(GetEffectiveSkill(caster,SKILLID_EVALINT));
    evalint:= CInt((evalint / 9) - RandomInt(evalint / 18));
    dmg:= dmg + evalint;
    ApplyRawDamage(cast_on, Resisted(circle,caster,cast_on, dmg));
  endif
endfunction

/////////////////////////
// Poison ///////////////
/////////////////////////
function cast_poison( caster, cast_on )
  var circle := 3;
  if((!GetStrength(cast_on)) || (cast_on.dead))
    return;
  endif
  SetObjProperty(cast_on, "LastHit", {caster.name,caster.serial, "poison spell" });
  if(Reflected(cast_on))
	cast_on := caster;
  endif
  PlaySoundEffect(cast_on, 0x206);
  PlayObjectCenteredEffect(cast_on, FX_CURSE_EFFECT, 0x0a,0x1e);
  if(cast_on.dead)
    return;
  endif
  var p_level :=  0;
  var skil := GetEffectiveSkill(caster, SKILLID_MAGERY);
  if(skil <= 50)
    p_level := 1;
  elseif(skil <= 98)
    p_level := 2;
  else
    p_level := 3;
  endif
  var poison_level := CInt(GetObjProperty(cast_on, "poison_level"));
  var passparms := { cast_on, caster, "poison spell" };
  var newamt;
  if(cast_on.isA(POLCLASS_NPC))
    var element := npccfg_sr[cast_on.npctemplate].elemental;
    if(element == "poison")
      HealDamage(cast_on, (p_level * 50));
    else
      newamt := poisonresist(circle, caster, cast_on, p_level);
      if(newamt > 0)
      	  SetObjProperty(cast_on, "poison_level", newamt);
          SendSysMessage(cast_on, "You have been poisoned by " + caster.name +"!");
          Detach();
          SetObjProperty(cast_on,"PoisonStamp", ReadGameClock());
          start_script(":spells:poisonDamage", passparms);
          SetObjProperty(cast_on, "LastHit", {caster.name, caster.serial, "poison spell" });
      endif
    endif
  else
    newamt := poisonresist(circle, caster, cast_on, p_level);
    if(newamt > 0)
        if(poison_level != error)
	  SetObjProperty(cast_on, "poison_level", CInt(newamt + poison_level));
	else
	  SetObjProperty(cast_on, "poison_level", newamt);
	endif
      SendSysMessage(cast_on, "You have been poisoned by " + caster.name +"!");
      Detach();
      SetObjProperty(cast_on,"PoisonStamp", ReadGameClock());
      start_script(":spells:poisonDamage", passparms);
      SetObjProperty(cast_on, "LastHit", {caster.name, caster.serial, "poison spell" });
    endif
  endif
endfunction

/////////////////////////
// Poison Resist ////////
/////////////////////////
function poisonresist(circle, caster, cast_on, amt)
  var evalint := CInt(GetEffectiveSkill(caster, SKILLID_EVALINT));
  var diff := CInt((circle * 10) + (evalint / 10));
  var points := (diff * 2);
  if(cast_on.dead)
    return 0;
  endif
  if(CheckSkill(cast_on, SKILLID_MAGICRESISTANCE, diff, points))
	SendSysMessage(cast_on, "You feel yourself resisting magical energy!");
	PlaySoundEffect(cast_on, SFX_SPELL_WEAKEN);
	PlayObjectCenteredEffect( cast_on, FX_CURSE_EFFECT, 5,5);
	if(amt == 3)
      amt := 1;
    elseif(amt == 2)
      amt := 1;
    elseif(amt == 1)
      amt := 0;
    endif
	return amt;
  else
	return amt;
  endif
endfunction

/////////////////////////
// Teleport /////////////
/////////////////////////
function cast_teleport(caster, cast_on)
  var sx := caster.x, sy := caster.y, sz := caster.z;
  var multicheck := CreateItemAtLocation(cast_on.x, cast_on.y, cast_on.z, 0xeed, 1);
  multicheck.hidden := 1;
  if (multicheck.multi.serial != error)
    DestroyItem(multicheck);
    return;
  else
    var items:= ListItemsNearLocation(cast_on.x, cast_on.y, cast_on.z,5);
    var count:= len(items);
    if(count > 5)
      count:= 5;
    endif
    while(count)
      if(items[RandomInt(len(items)) + 1].multi.serial != error)
        DestroyItem(multicheck);
        return;
      endif
      count:= count - 1;
    endwhile
    if((multicheck.x > 5120) && (multicheck.x < 6133) && (multicheck.y > 2063) && (multicheck.y < 2303))
      DestroyItem(multicheck);
      return;
    endif
    DestroyItem(multicheck);
    MoveCharacterToLocation( caster, cast_on.x, cast_on.y, cast_on.z );
    PlayStationaryEffect( sx, sy, sz, FX_SMOKE, speed:=0xa, loop:=0xa );
    PlayStationaryEffect( caster.x, caster.y, caster.z, FX_SMOKE, speed:=0xa, loop:=0xa );
    PlaySoundEffect( caster, SFX_SPELL_TELEPORT );
  endif
endfunction

/////////////////////////
// Arch Cure ////////////
/////////////////////////
function cast_archcure( caster, cast_on )
  PlaySoundEffect(caster, SFX_SPELL_ARCH_CURE);
  var magery := GetEffectiveSkill(caster, SKILLID_MAGERY);
  var poison_descr;
  var poison_level;
  var potency := CInt(GetEffectiveSkill( caster, SKILLID_MAGERY ) /5);
  foreach cast_on in ListMobilesNearLocation( cast_on.x, cast_on.y, cast_on.z, CInt(magery/15) );
    if (CheckLineOfSight(cast_on, caster) )
	poison_descr := GetObjProperty( cast_on,"poison_level" );
	poison_level := CInt(poison_descr);
	PlayObjectCenteredEffect( cast_on, FX_BLESS_EFFECT, 10,10);
	if(cast_on.isA(POLCLASS_NPC))
	  var element := npccfg_sr[cast_on.npctemplate].elemental;
	  if(element == "poison")
	    var ev := array;
	    ev.+ type;
	    ev.+ source;
	    ev.type := EVID_PEACEMADE;
	    SendEvent(cast_on, ev);
	    ev.source := caster;
	    SendEvent(cast_on, ev);
	    ev.type := EVID_ENGAGED;
	    SendEvent(cast_on, ev);
	    ApplyRawDamage(cast_on, potency);
	  else
	    if (poison_level >= 1)
	      poison_level := poison_level - potency;
	      if (poison_level >= 1)
		SendSysMessage(cast_on, caster.name + " has partially cured you!");
	      else
		SendSysMessage(cast_on, caster.name + " has cured you completely!");
	      endif
	    SetObjProperty(cast_on, "poison_level", CStr(poison_level));
	    endif
	  endif
	else
	  if (poison_level >= 1)
	    poison_level := poison_level - potency;
	    if (poison_level >= 1)
	      SendSysMessage(cast_on, caster.name + " has partially cured you!");
	    else
	      SendSysMessage(cast_on, caster.name + " has cured you completely!");
	    endif
	    SetObjProperty(cast_on, "poison_level", CStr(poison_level));
	  endif
	endif
    endif
  endforeach
endfunction

/////////////////////////
// Arch Protection //////
/////////////////////////
function cast_archprotection( caster, cast_on )
  PlaySoundEffect(caster, SFX_SPELL_PROTECTION);
  var magery := GetEffectiveSkill(caster, SKILLID_MAGERY);
  var mod_amount := CInt(RandomInt(6) + 5);
  var duration := GetModDuration( magery );
  foreach cast_on in ListMobilesNearLocation( cast_on.x, cast_on.y, cast_on.z, CInt(magery/15) );
	if (CanMod(cast_on, "ar"))
	  PlayObjectCenteredEffect( cast_on, FX_BLESS_EFFECT, 7,7);
	  DoTempMod(cast_on, "ar", mod_amount, duration);
	endif
  endforeach
endfunction

/////////////////////////
// Curse ////////////////
/////////////////////////
function cast_curse( caster, cast_on )
  PlaySoundEffect(cast_on, SFX_SPELL_CURSE);
  PlayObjectCenteredEffect( cast_on, FX_CURSE_EFFECT, 10,10);
  var magery := GetEffectiveSkill(caster, SKILLID_MAGERY);
  var mod_amount := Resisted(4,caster,cast_on,GetModAmount( magery ));
  var duration := GetModDuration( magery );
	SetObjProperty(cast_on, "LastHit", {caster.name,caster.serial, "curse" });
  if(Reflected(cast_on))
	cast_on := caster;
  endif
  if (CanMod(cast_on, "str"))
	DoTempMod(cast_on, "str", 0- mod_amount, duration);
  endif
  if (CanMod(cast_on, "dex"))
	DoTempMod(cast_on, "dex", 0- mod_amount, duration);
  endif
  if (CanMod(cast_on, "int"))
	DoTempMod(cast_on, "int", 0- mod_amount, duration);
  endif
endfunction

/////////////////////////
// Greater Heal /////////
/////////////////////////
function cast_greaterheal( caster, cast_on )
  var healed := RandomDiceRoll("1d3");
  healed := CInt(GetEffectiveSkill(caster, SKILLID_MAGERY)/2) + healed;
  if(cast_on.isA(POLCLASS_NPC))
    healed := healed * 2;
  endif
  var evalint := Cint(GetEffectiveSkill(caster, SKILLID_EVALINT) / 10);
  healed := healed + evalint;
  HealDamage(cast_on, healed);
  PlayObjectCenteredEffect( cast_on, FX_SPARK_EFFECT, 7, 0x10 );
  PlaySoundEffect( cast_on, SFX_SPELL_GREATER_HEAL );
endfunction

/////////////////////////
// Lightning /////////////
/////////////////////////
function cast_lightning( caster, cast_on )
  var circle := 4;
  var dmg := RandomDiceRoll("3d7");
  if (dmg < 1)
	dmg := 1;
  endif
  PlaySoundEffect(cast_on, SFX_SPELL_LIGHTNING);
  PlayLightningBoltEffect( cast_on);
  if(cast_on.isA(POLCLASS_NPC))
    dmg := dmg * 2;
  endif
  SetObjProperty(cast_on, "LastHit", {caster.name,caster.serial, "lightning bolt"});
  if(Reflected(cast_on))
	cast_on := caster;
    PlaySoundEffect(cast_on, SFX_SPELL_LIGHTNING);
    PlayLightningBoltEffect( cast_on);
  endif
  var evalint:= CInt(GetEffectiveSkill(caster,SKILLID_EVALINT));
  evalint:= CInt((evalint / 8) - RandomInt(evalint / 16));
  dmg:= dmg + evalint;
  ApplyRawDamage(cast_on, Resisted(circle,caster,cast_on,dmg));
endfunction

/////////////////////////
// Mana Drain ///////////
/////////////////////////
function cast_manadrain( caster, cast_on )
  var circle := 4;
  var caster_magery := GetEffectiveSkill( caster, SKILLID_MAGERY );
  SetObjProperty(cast_on, "LastHit", {caster.name,caster.serial, "mana drain" });
  if (Reflected(cast_on))
	var tmp := cast_on;
	cast_on := caster;
	caster := tmp;
  endif
  var dmg := RandomInt(caster_magery / 5) + 8;
  var evalint := Cint(GetEffectiveSkill(caster, SKILLID_EVALINT) / 10);
  dmg := dmg + evalint;
  var diff := Resisted(circle, caster, cast_on, dmg);
  if(diff)
    PlaySoundEffect( caster, SFX_SPELL_MANA_DRAIN );
    PlayObjectCenteredEffect( cast_on, FX_HEAL_EFFECT, 7,0x10 );
	if(diff > GetMana(cast_on))
	  SetMana(cast_on, 0);
	else
	  SetMana(cast_on, GetMana(cast_on) - diff);
	endif
  endif
endfunction

/////////////////////////
// Dispel Field//////////
/////////////////////////
function cast_dispelfield( caster, cast_on )
  var magery := GetEffectiveSkill(caster, SKILLID_MAGERY);
  var therange := CInt(magery / 15 );
  var items := ListItemsNearLocation(cast_on.x, cast_on.y, cast_on.z, therange);
  PlaySoundEffect( caster, SFX_SPELL_DISPEL );
  foreach item in items
	if (GetObjProperty(item, "f") == "1")
	  PlayStationaryEffect( item.x, item.y, item.z, FX_SMOKE, 0xa, 0xa );
      DestroyItem(item);
	endif
  endforeach
endfunction

/////////////////////////
// Magic Reflection /////
/////////////////////////
function cast_reflect( caster )

  if (!(GetObjProperty(caster, "mr") == "1") )
	PlaySoundEffect(caster, 0x1ea);
	PlayObjectCenteredEffect( caster, FX_SPARK_EFFECT, 10,10);
	SetObjProperty(caster, "mr","1");
  else
    PlaySoundEffect(caster,0x005d);
    PlayObjectCenteredEffect(caster, 0x3735,0x0a, 0x1e);
  endif
endfunction

/////////////////////////
// Mind Blast ///////////
/////////////////////////
function cast_mindblast( caster, cast_on )
  var circle := 5;
  var my_int := CInt(GetIntelligence(caster));
  var his_int := CInt(GetIntelligence(cast_on));
  SetObjProperty(cast_on, "LastHit", {caster.name,caster.serial, "mind blast" });
  var modif := my_int - his_int;
  if(his_int > my_int)
    modif := his_int - my_int;
	cast_on := caster;
  elseif(his_int == my_int)
	return;
  endif
  if(modif > 20)
    modif:= 20;
  endif
  var dmg := RandomDiceRoll("1d7") + modif;
  var diff := Resisted(circle, caster, cast_on, dmg);
  PlayObjectCenteredEffect(cast_on, FX_CURSE_EFFECT, 7,0x10);

  if(Reflected(cast_on))
	cast_on := caster;
  endif
  PlaySoundEffect(cast_on, SFX_SPELL_MIND_BLAST);
  if(cast_on.isA(POLCLASS_NPC))
    diff := diff * 2;
  endif
  ApplyRawDamage(cast_on, diff);
endfunction

/////////////////////////
// Paralyze /////////////
/////////////////////////
function cast_paralyze( caster, cast_on )

  SetObjProperty(cast_on, "LastHit", {caster.name,caster.serial, "paralyze" });
  if (Reflected(cast_on))
    cast_on := caster;
  endif
  var duration := CInt(pararesist(3, caster, cast_on));
  PlayStationaryEffect( cast_on.x, cast_on.y, cast_on.z, FX_CURSE_EFFECT, 10, 0xa, 0 );
  PlaySoundEffect( cast_on, SFX_SPELL_PARALYZE );
  DoTempMod(cast_on, "p", 1, duration);
endfunction

/////////////////////////
// Paralyze Resist //////
/////////////////////////
function pararesist(circle, caster, cast_on)
  var magery := CInt(GetEffectiveSkill(caster, SKILLID_MAGERY));
  var evalint := CInt(GetEffectiveSkill(caster, SKILLID_EVALINT));
  var resist := CInt(GetEffectiveSkill(cast_on, SKILLID_MAGICRESISTANCE));
  var chk1 := (resist / 5);
  var chk2 := resist - (((magery - 20) / 5) + (circle * 5));
  if(chk1 < 1)
    chk1 := 1;
  endif
  if(chk2 < 1)
    chk2 := 1;
  endif
  var diff := 0;
  if(chk1 > chk2)
    diff := chk1;
  else
    diff := chk2;
  endif
  var points := getresistpoints(circle, cast_on, SKILLID_MAGICRESISTANCE);
  if(cast_on.dead)
    return 0;
  endif
  var amt;
  if((RandomInt(99) + 1) <= diff)
	SendSysMessage(cast_on, "You feel yourself resisting magical energy!");
	PlaySoundEffect(cast_on, SFX_SPELL_WEAKEN);
	PlayObjectCenteredEffect(cast_on, FX_CURSE_EFFECT, 5,5);
	AwardRawSkillPoints(cast_on, SKILLID_MAGICRESISTANCE, points);
	amt := RandomInt(3) + 1;
	return amt;
  endif
  amt := CInt(evalint / 20 + 2);
  amt := CInt(amt - RandomInt(resist / 20));
  return amt;
endfunction

/////////////////////////
// Dispel ///////////////
/////////////////////////
function cast_dispel( caster, cast_on )
  var circle := 6;
  var summoned := GetObjProperty( cast_on, "summoned");
  if(summoned == 1)
    PlaySoundEffect( caster, 0x202 );
    PlayStationaryEffect( cast_on.x, cast_on.y, cast_on.z, FX_HEAL_EFFECT, 0xa, 0xa );
    var master := SystemFindObjectBySerial(CInt(GetObjProperty(cast_on, "master")));
    if(master)
      var resist := GetEffectiveSkill(master, SKILLID_MAGICRESISTANCE);
      send_attack(caster, cast_on, 41);
      if((RandomInt(99) + 1) < resist)
        MoveCharacterToLocation(cast_on,5275,1166,0,MOVECHAR_FORCELOCATION);
        Damage(cast_on, GetMaxHp(cast_on) + 3);
      endif
    endif
   else
    if(cast_on.isA(POLCLASS_NPC))
      var element := npccfg_sr[cast_on.npctemplate].elemental;
      if(element == "undead")
        PlayStationaryEffect( cast_on.x, cast_on.y, cast_on.z, FX_HEAL_EFFECT, 0xa, 0xa );
        var ev := array;
        ev.+ type;
        ev.+ source;
        ev.type := EVID_PEACEMADE;
        SendEvent(cast_on, ev);
        ev.source := caster;
        SendEvent(cast_on, ev);
        ev.type := EVID_ENGAGED;
        SendEvent(cast_on, ev);
        PlaySoundEffect(caster, 0x202);
        PlayObjectCenteredEffect(cast_on, 0x37b9, 0x03, 0x0a);
        var dmg := CalcSpellDamage(circle, caster, cast_on) + (RandomInt(20));
        ApplyRawDamage(cast_on, Resisted(circle, caster, cast_on, dmg));
      endif
    endif 
  endif
endfunction

/////////////////////////
// Energy Bolt //////////
/////////////////////////
function cast_energybolt( caster, cast_on )
  var circle := 6;
  if(!cast_on.container)
    PlayMovingEffect( caster, cast_on, FX_ENERGY_BOLT, 0xa, 0x1e );
  endif
  PlaySoundEffect( caster, SFX_SPELL_ENERGY_BOLT );
  SetObjProperty(cast_on, "LastHit", {caster.name, caster.serial, "ebolt" } );
  if(Reflected(cast_on))
	cast_on := caster;
    PlayMovingEffect( caster, cast_on, FX_ENERGY_BOLT, 0xa, 0x1e );
    PlaySoundEffect( caster, SFX_SPELL_ENERGY_BOLT );
  endif
  var dmg := RandomDiceRoll("5d8");
  if (Reflected(cast_on))
	PlayMovingEffect( cast_on, caster, FX_ENERGY_BOLT, 0xa, 0x1e );
	PlaySoundEffect( cast_on, SFX_SPELL_ENERGY_BOLT );
	cast_on := caster;
	if (GetObjProperty( caster, "mr") == "1")
	  EraseObjProperty(caster, "mr");
	  return;
	endif
  endif
  if(cast_on.isA(POLCLASS_NPC))
    dmg := dmg * 2;
  endif
  var evalint:= CInt(GetEffectiveSkill(caster,SKILLID_EVALINT));
  evalint:= CInt((evalint / 7) - RandomInt(evalint / 14));
  dmg:= dmg + evalint;
  ApplyRawDamage(cast_on, Resisted(circle, caster, cast_on, dmg));
endfunction

/////////////////////////
// Explosion ////////////
/////////////////////////
function cast_explosion( caster, cast_on )
  var circle := 6;
  var dmg;
  SetObjProperty(cast_on, "LastHit", ( {caster.name, caster.serial, "explosion" } ) );
  if(Reflected(cast_on))
	cast_on := caster;
  endif
  dmg := RandomDiceRoll("7d6");
  Detach();
  send_attack(cast_on,caster, 43);
  PlaySoundEffect( cast_on, SFX_SPELL_EXPLOSION );
  PlayStationaryEffect( cast_on.x,cast_on.y,cast_on.z, FX_EXPLODE_3, 7, 0x10 );
  if(GetHp(cast_on) > 0)
    if(cast_on.npctemplate)
      dmg := dmg * 2;
    endif
    var evalint:= CInt(GetEffectiveSkill(caster,SKILLID_EVALINT));
    evalint:= CInt((evalint / 5) - RandomInt(evalint / 10));
    dmg:= dmg + evalint;
    ApplyRawDamage(cast_on, Resisted(circle,caster,cast_on,dmg));
  endif
endfunction

/////////////////////////
// Invisibility /////////
/////////////////////////
function cast_invisibility( caster, cast_on )
  PlaySoundEffect(cast_on, SFX_SPELL_INVISIBILITY);
  cast_on.hidden := 1;
  Detach();
  sleep(CInt(GetEffectiveSkill(caster, SKILLID_MAGERY)));
  cast_on.hidden := 0;
endfunction

/////////////////////////
// Mass Curse ///////////
/////////////////////////
function cast_masscurse( caster, cast_on )
  var circle := 6;
  PlaySoundEffect(caster, SFX_SPELL_MASS_CURSE);
  var magery := GetEffectiveSkill(caster, SKILLID_MAGERY);
  var mod_amount := GetModAmount( magery );
  var duration := GetModDuration( magery );
  Detach();
  foreach cast_on in ListMobilesNearLocation( cast_on.x, cast_on.y, cast_on.z, 4 );
	SetObjProperty(cast_on, "LastHit", {caster.name, caster.serial, "mass curse"});
    if(Reflected(cast_on))
	  cast_on := caster;
    endif
    PlaySoundEffect(cast_on, SFX_SPELL_MASS_CURSE);
    PlayObjectCenteredEffect( cast_on, FX_CURSE_EFFECT, 10,10);
	send_attack(cast_on,caster, 46);
    if(cast_on.criminal == 0)
      caster.setCriminal(1);
    endif
    mod_amount := Resisted(circle,caster,cast_on, GetModAmount( magery ));
	if (CanMod(cast_on, "str"))
	  DoTempMod(cast_on, "str", 0- mod_amount, duration);
	endif
	if (CanMod(cast_on, "dex"))
	  DoTempMod(cast_on, "dex", 0- mod_amount, duration);
	endif
	if (CanMod(cast_on, "int"))
	  DoTempMod(cast_on, "int", 0- mod_amount, duration);
	endif
    endforeach
endfunction

/////////////////////////
// Reveal ///////////////
/////////////////////////
function cast_reveal( caster, cast_on )
  var magic_skill := Resisted(6, caster, cast_on, GetEffectiveSkill( caster, SKILLID_MAGERY ));
  var resist := (GetEffectiveSkill( cast_on, SKILLID_MAGICRESISTANCE)/1.5);
  var range := ( GetEffectiveSkill( caster, SKILLID_MAGERY  )/10 ) - 3;
  if (range < 1)
	range := 1;
  endif
  var hiders := ListMobilesNearLocationEx( caster.x, caster.y, caster.z, range, LISTEX_FLAG_HIDDEN );
  var numfound := 0;
  PlaySoundEffect(caster, SFX_1FE);
  if(len(hiders) > 0)
	foreach mobile in hiders
	  if( (magic_skill/2) > resist )
		if(mobile.npctemplate != "playervendor")
		  numfound := 1;
		  mobile.hidden := 0;
		  PrintTextAbovePrivate( mobile, "You have been revealed!", mobile );
		  PlaySoundEffect(mobile, SFX_1FE);
		endif
	  endif
	endforeach
  endif
endfunction

/////////////////////////
// Chain Lightning ///////
/////////////////////////
function cast_chainlightning( caster, cast_on )
  var circle := 7;
  var dmg;
  var range := 2;
  var victims := ListMobilesNearLocationEx( cast_on.x, cast_on.y, cast_on.z, range, 0x03);
  foreach mobile in victims
	SetObjProperty(mobile, "LastHit", {caster.name, caster.serial, "chain lightning"});
    if(Reflected(mobile))
	  mobile := caster;
    endif
	dmg := RandomDiceRoll("6d8");
	PlayLightningBoltEffect( mobile );
	PlaySoundEffect(mobile, SFX_SPELL_LIGHTNING);
    if(mobile.isA(POLCLASS_NPC))
      dmg := dmg * 2;
    endif
    var evalint:= CInt(GetEffectiveSkill(caster,SKILLID_EVALINT));
    evalint:= CInt((evalint / 7) - RandomInt(evalint / 14));
    dmg:= dmg + evalint;
    ApplyRawDamage( mobile, Resisted(circle,caster,mobile,dmg) );
    sleepms(600);
  endforeach
endfunction

/////////////////////////
// Flamestrike //////////
/////////////////////////
function cast_flamestrike( caster, cast_on )
  var circle := 7;
  var dmg := RandomDiceRoll("7d8");
  SetObjProperty(cast_on, "LastHit", {caster.name,caster.serial, "flame strike" });
  if (Reflected(cast_on))
	cast_on := caster;
  endif
  PlayObjectCenteredEffect(cast_on, 0x3709, 0x0a, 0x1e );
  PlaySoundEffect( caster, SFX_SPELL_FLAME_STRIKE );
  if(GetHp(cast_on) >= 1)
    if(cast_on.npctemplate)
      dmg := dmg * 2;
    endif
    var evalint:= CInt(GetEffectiveSkill(caster,SKILLID_EVALINT));
    evalint:= CInt((evalint / 5) - RandomInt(evalint / 10));
    dmg:= dmg + evalint;
    ApplyRawDamage(cast_on, Resisted(circle,caster,cast_on,dmg));
  endif
endfunction

/////////////////////////
// Mana Vampire /////////
/////////////////////////
function cast_manavampire( caster, cast_on )
  var circle := 7;
  var caster_magery := GetEffectiveSkill( caster, SKILLID_MAGERY );
  SetObjProperty(cast_on, "LastHit", {caster.name,caster.serial, "mana vampire" });
  if (Reflected(cast_on))
	cast_on := caster;
  endif
  if(!cast_on.isA(POLCLASS_MOBILE))
    return;
  endif
  var dmg := RandomInt(caster_magery / 5) + 8;
  var evalint := Cint(GetEffectiveSkill(caster, SKILLID_EVALINT) / 10);
  dmg := dmg + evalint;
  var diff := Resisted(circle, caster, cast_on, dmg);
  if(diff)
    PlaySoundEffect(caster, SFX_SPELL_MANA_VAMPIRE);
    PlayObjectCenteredEffect(cast_on, FX_GLOW, 7,0x10);
    var gain := 0;
	if(diff > GetMana(cast_on))
	  gain := GetMana(cast_on);
	  SetMana(cast_on, 0);
	else
	  gain := diff;
	  SetMana(cast_on, GetMana(cast_on) - diff);
	endif
	SetMana(caster, GetMana(caster) + gain);
	if(GetMana(caster) > GetIntelligence(caster))
	  SetMana(caster, caster.intelligence);
	endif
  endif
endfunction

/////////////////////////
// Mass Dispel //////////
/////////////////////////
function cast_massdispel( caster, cast_on )
  var circle := 7;
  var magery := GetEffectiveSkill(caster, SKILLID_MAGERY);
  var range := CInt(magery / 20);
  var targets := ListMobilesNearLocation( cast_on.x, cast_on.y, cast_on.z, range );
  PlaySoundEffect( caster, 0x202 );
  foreach mobile in targets
	SetObjProperty(mobile, "LastHit", {caster.name, caster.serial, "mass dispel"});
    if(Reflected(mobile))
	  mobile := caster;
    endif
    var summoned := GetObjProperty( mobile, "summoned");
	if (summoned == 1)
      PlaySoundEffect(caster, 0x202 );
      PlayStationaryEffect( mobile.x, mobile.y, mobile.z, FX_HEAL_EFFECT, 0xa, 0xa );
      var master := SystemFindObjectBySerial(CInt(GetObjProperty(mobile, "master")));
      if(master)
        var resist := GetEffectiveSkill(master, SKILLID_MAGICRESISTANCE);
        if((RandomInt(99) + 1) < resist)
          MoveCharacterToLocation(mobile,0,0,0,MOVECHAR_FORCELOCATION);
          Damage(mobile, GetMaxHp(mobile) + 3);
        endif
        send_attack(caster, mobile, 41);
    endif
    else
      if(mobile.isA(POLCLASS_NPC))
        var element := npccfg_sr[mobile.npctemplate].elemental;
        if(element == "undead")
          PlayStationaryEffect( mobile.x, mobile.y, mobile.z, FX_HEAL_EFFECT, 0xa, 0xa );
          var ev := array;
          ev.+ type;
          ev.+ source;
          ev.type := EVID_PEACEMADE;
          SendEvent(mobile, ev);
          ev.source := caster;
          SendEvent(mobile, ev);
          ev.type := EVID_ENGAGED;
          SendEvent(mobile, ev);
	      PlaySoundEffect( caster, 0x202 );
          PlayObjectCenteredEffect(mobile, 0x37b9,0x03, 0x0a);
          var dmg := CalcSpellDamage(circle,caster,mobile) + (RandomInt(20));
          ApplyRawDamage(mobile, Resisted(circle,caster,mobile,dmg));
        endif
      endif
    endif
  endforeach
endfunction

/////////////////////////
// Meteor Swarm /////////
/////////////////////////
function cast_meteor( caster, cast_on )
  var circle := 7;
  var dmg;
  var victims := ListMobilesNearLocationEx( cast_on.x, cast_on.y, cast_on.z, 3, 0x03);
  foreach mobile in victims
	PlaySoundEffect( mobile, SFX_SPELL_FIREBALL );
	PlayMovingEffect( caster, mobile, FX_FIREBALL, 5, 1 );
	SetObjProperty(mobile, "LastHit", {caster.name, caster.serial, "meteor swarm"});
    if(Reflected(mobile))
	  mobile := caster;
	  PlaySoundEffect( mobile, SFX_SPELL_FIREBALL );
	  PlayMovingEffect( caster, mobile, FX_FIREBALL, 5, 1 );
    endif
	dmg := RandomDiceRoll("6d8");
	send_attack(mobile,caster, 55);
    if(GetHp(mobile) > 1)
      if(cast_on.npctemplate)
        dmg := dmg * 2;
      endif
      var evalint:= CInt(GetEffectiveSkill(caster,SKILLID_EVALINT));
      evalint:= CInt((evalint / 7) - RandomInt(evalint / 14));
      dmg:= dmg + evalint;
      ApplyRawDamage(mobile, Resisted(circle,caster,mobile,dmg));
    endif
    sleep(1);
  endforeach
endfunction

/////////////////////////
// Polymorph ////////////
/////////////////////////
function cast_polymorph( caster )
  var origform := caster.graphic;
  var magery     := CInt(GetEffectiveSkill(caster, SKILLID_MAGERY));
  var meditation := CInt(GetEffectiveSkill(caster, SKILLID_MEDITATION));
  var evalint    := CInt(GetEffectiveSkill(caster, SKILLID_EVALINT));
  var critgroup;
  var thecreature;
  if (magery < 70)
	critgroup := RandomInt(1);
  elseif (magery < 80)
	critgroup := RandomInt(1)+1;
  elseif (magery < 90)
	critgroup := RandomInt(1)+2;
  else
	critgroup := 3;
  endif
  var critter := RandomInt(8);
  case (critgroup)
	0:
	  case (critter)
		0: thecreature := 6;	//bird
		1: thecreature := 51;	//slime
		2: thecreature := 5;	//eagle
		3: thecreature := 39;	//mongbat
		4: thecreature := 31;	//headless
		5: thecreature := 29;	//gorilla
		6: thecreature := 42;	//ratman
		7: thecreature := 28;	//giant spider
	  endcase
	1:
	  case (critter)
		0: thecreature := 48;	//scorpion
		1: thecreature := 7;	//orc
		2: thecreature := 3;	//zombie
		3: thecreature := 17;	//orc
		4: thecreature := 33;	//lizardman
		5: thecreature := 26;	//shade
		6: thecreature := 21;	//giant serp
		7: thecreature := 30;	//harpy
	  endcase
	2:
	  case (critter)
	    0: thecreature := 18;	//ettin
		1: thecreature := 8;	//corpser
		2: thecreature := 22;	//beholder
		3: thecreature := 14;	//earth elem
		4: thecreature := 16;	//water elem
		5: thecreature := 15;	//fire elem
		6: thecreature := 13;	//air elem
		7: thecreature := 47;	//ent
	  endcase
	3:
	  case (critter)
		0: thecreature := 1;	//ogre
		1: thecreature := 4;	//gargoyle
		2: thecreature := 24;	//lich
		3: thecreature := 75;	//titan    
		4: thecreature := 9;	//demon
		5: thecreature := 12;	//dragon
		6: thecreature := 10;	//demon with sword
		7: thecreature := 58;	//wisp
	  endcase
  endcase
  caster.graphic := thecreature;
  if ( thecreature == 150 )
	caster.color := 233;
  else
	caster.color := 0;
  endif
  var magerydur := magery/2;
  var meditadur := magerydur + meditation/10;
  var duration  := meditadur + evalint/10;
  Detach();
  sleep(duration);
  caster.graphic := origform;
  caster.color := caster.truecolor;
  EraseObjProperty(caster,"poly");
endfunction

/////////////////////////
// Earthquake ///////////
/////////////////////////
function cast_earthquake( caster )
  var circle := 8;
  var range := 12;
  var victims := ListMobilesNearLocationEx( caster.x, caster.y, caster.z, range, LISTEX_FLAG_NORMAL + LISTEX_FLAG_HIDDEN );
  var dmg;
  PlaySoundEffect( caster, SFX_220 );
  foreach mobile in victims
    if( mobile != caster )
      SetObjProperty(mobile, "LastHit", {caster.name,caster.serial, "earthquake" });
      send_attack(mobile,caster, 57);
      dmg := RandomDiceRoll("7d7");
	  PrintTextAbovePrivate( mobile,"You are tossed about as the earth trembles below your feet!", mobile );
      PlaySoundEffect( mobile, SFX_SPELL_EARTHQUAKE );
	  PlayObjectCenteredEffect( mobile, FX_SMOKE, 7, 7);
      if(mobile.isA(POLCLASS_NPC))
        dmg := dmg * 2;
      endif
      var evalint:= CInt(GetEffectiveSkill(caster,SKILLID_EVALINT));
      evalint:= CInt((evalint / 6) - RandomInt(evalint / 12));
      dmg:= dmg + evalint;
      ApplyRawDamage( mobile, Resisted(circle,caster,mobile,dmg) );
    endif
  endforeach
  sleep(2);
  PlaySoundEffect( caster, SFX_223 );
endfunction

/////////////////////////
// Summon vortex ////////
/////////////////////////
function  cast_summonvortex(caster)
    var utterance := RandomInt(4);
    case (utterance)
        0: AI_Speak(caster, "Lightning flash and fire burn!");
        1: AI_Speak(caster, "Awake, oh servant of chaos!");
        2: AI_Speak(caster, "Rise up and consume my enemies, slave!");
        3: AI_Speak(caster, "Come forth, Chaos spawn!");
    endcase
    var thecritter := CreateNpcFromTemplate("vortex", caster.x, caster.y -2, caster.z, 0);
    PlaySoundEffect(thecritter,SFX_SPELL_ENERGY_VORTEX);
    SetObjProperty(thecritter, "summoned",1);
    SetObjProperty(thecritter, "master", caster.serial);
    Detach();
    sleep( (GetEffectiveSkill( caster, SKILLID_MAGERY ) ) );
    PlaySoundEffect( thecritter, SFX_SPELL_MANA_VAMPIRE );
    PlayStationaryEffect( thecritter.x, thecritter.y, thecritter.z, FX_SMOKE, 0xa, 0xa );
    MoveCharacterToLocation(thecritter,0,0,0,MOVECHAR_FORCELOCATION);
    ApplyRawDamage(thecritter, GetHp(thecritter) + 3);
endfunction

/////////////////////////
// Summon air ///////////
/////////////////////////
function  cast_summonair(caster)
    var utterance := RandomInt(4);
    case (utterance)
        0: AI_Speak(caster, "Wind blow and lightning flash! I call upon the forces of nature!");
        1: AI_Speak(caster, "Do you like wind, human?");
        2: AI_Speak(caster, "I call uppon the forces of wind!");
        3: AI_Speak(caster, "By the winds of the east and west, I command you to serve me");
    endcase
    var thecritter := CreateNpcFromTemplate("airelemental", caster.x, caster.y -2, caster.z, 0);
    PlaySoundEffect(thecritter,SFX_SPELL_SUMMON_ELEMENTAL);
    SetObjProperty(thecritter, "summoned",1);
    SetObjProperty(thecritter, "master", caster.serial);
    Detach();
    sleep( (GetEffectiveSkill( caster, SKILLID_MAGERY ) ) );
    PlaySoundEffect( thecritter, SFX_SPELL_MANA_VAMPIRE );
    PlayStationaryEffect( thecritter.x, thecritter.y, thecritter.z, FX_SMOKE, 0xa, 0xa );
    MoveCharacterToLocation(thecritter,0,0,0,MOVECHAR_FORCELOCATION);
    ApplyRawDamage(thecritter, GetHp(thecritter) + 3);
endfunction

/////////////////////////
// Summon daemon ////////
/////////////////////////
function  cast_summondaemon(caster)
    var utterance := RandomInt(4);
    case (utterance)
        0: AI_Speak(caster, "Come forth, servant, and destroy these mortal fools!");
        1: AI_Speak(caster, "Servant, I command you: Destroy these mortals");
        2: AI_Speak(caster, "Fire and hail, blood and brimstone, heed my command!");
        3: AI_Speak(caster, "Take these slaves back to your realm, servant of darkness!");
    endcase
    var thecritter := CreateNpcFromTemplate("summoneddaemon", caster.x, caster.y -2, caster.z, 0);
    PlaySoundEffect(thecritter,SFX_SPELL_SUMMON_DAEMON);
    SetObjProperty(thecritter, "summoned",1);
    SetObjProperty(thecritter, "master", caster.serial);
    Detach();
    sleep( (GetEffectiveSkill( caster, SKILLID_MAGERY ) ) );
    PlaySoundEffect( thecritter, SFX_SPELL_MANA_VAMPIRE );
    PlayStationaryEffect( thecritter.x, thecritter.y, thecritter.z, FX_SMOKE, 0xa, 0xa );
    MoveCharacterToLocation(thecritter,0,0,0,MOVECHAR_FORCELOCATION);
    ApplyRawDamage(thecritter, GetHp(thecritter) + 3);
endfunction

/////////////////////////
// Summon earth /////////
/////////////////////////
function  cast_summonearth(caster)
    var utterance := RandomInt(4);
    case (utterance)
        0: AI_Speak(caster, "Awake, Son of Earth!");
        1: AI_Speak(caster, "Mud and stone, arise and walk!");
        2: AI_Speak(caster, "I call upon the stones and earth to drive forth my foes!");
        3: AI_Speak(caster, "Return to earth, human!");
    endcase
    var thecritter := CreateNpcFromTemplate("earthelemental", caster.x, caster.y -2, caster.z, 0);
    PlaySoundEffect(thecritter,SFX_SPELL_SUMMON_ELEMENTAL);
    SetObjProperty(thecritter, "summoned",1);
    SetObjProperty(thecritter, "master", caster.serial);
    Detach();
    sleep( (GetEffectiveSkill( caster, SKILLID_MAGERY ) ) );
    PlaySoundEffect( thecritter, SFX_SPELL_MANA_VAMPIRE );
    PlayStationaryEffect( thecritter.x, thecritter.y, thecritter.z, FX_SMOKE, 0xa, 0xa );
    MoveCharacterToLocation(thecritter,0,0,0,MOVECHAR_FORCELOCATION);
    ApplyRawDamage(thecritter, GetHp(thecritter) + 3);
endfunction

/////////////////////////
// Summon fire //////////
/////////////////////////
function  cast_summonfire(caster)
    var utterance := RandomInt(4);
    case (utterance)
        0: AI_Speak(caster, "Fire! I summon fire, death, and pain! Rise up, O Flame!");
        1: AI_Speak(caster, "Your ashes shall grace our altar, human!");
        2: AI_Speak(caster, "I call uppon the forces of fire!");
        3: AI_Speak(caster, "Burn, human, burn!");
    endcase
    var thecritter := CreateNpcFromTemplate("fireelemental", caster.x, caster.y -2, caster.z, 0);
    PlaySoundEffect(thecritter,SFX_SPELL_SUMMON_ELEMENTAL);
    SetObjProperty(thecritter, "summoned",1);
    SetObjProperty(thecritter, "master", caster.serial);
    Detach();
    sleep( (GetEffectiveSkill( caster, SKILLID_MAGERY ) ) );
    PlaySoundEffect( thecritter, SFX_SPELL_MANA_VAMPIRE );
    PlayStationaryEffect( thecritter.x, thecritter.y, thecritter.z, FX_SMOKE, 0xa, 0xa );
    MoveCharacterToLocation(thecritter,0,0,0,MOVECHAR_FORCELOCATION);
    ApplyRawDamage(thecritter, GetHp(thecritter) + 3);
endfunction

/////////////////////////
// Summon water /////////
/////////////////////////
function  cast_summonwater(caster)
    var utterance := RandomInt(4);
    case (utterance)
        0: AI_Speak(caster, "I call upon the powers of water and ice!");
        1: AI_Speak(caster, "Wash away my enemies, my pet");
        2: AI_Speak(caster, "Flow and conquer, servants");
        3: AI_Speak(caster, "Care for a drink, hummie?");
    endcase
    var thecritter := CreateNpcFromTemplate("summonelemental", caster.x, caster.y -2, caster.z, 0);
    PlaySoundEffect(thecritter,SFX_SPELL_SUMMON_ELEMENTAL);
    SetObjProperty(thecritter, "summoned",1);
    SetObjProperty(thecritter, "master", caster.serial);
    Detach();
    sleep( (GetEffectiveSkill( caster, SKILLID_MAGERY ) ) );
    PlaySoundEffect( thecritter, SFX_SPELL_MANA_VAMPIRE );
    PlayStationaryEffect( thecritter.x, thecritter.y, thecritter.z, FX_SMOKE, 0xa, 0xa );
    MoveCharacterToLocation(thecritter,0,0,0,MOVECHAR_FORCELOCATION);
    ApplyRawDamage(thecritter, GetHp(thecritter) + 3);
endfunction

/////////////////////////
// Summon skeleton //////
/////////////////////////
function  cast_summonskel(caster)
  if(Getmana(caster) >= 20)
    var utterance := RandomInt(4);
    case (utterance)
        0: AI_Speak(caster, "Awake, my children! Come forth and take your rightful place!");
        1: AI_Speak(caster, "Come forth my minions!");
        2: AI_Speak(caster, "I call upon the forces of darkness to attend me!");
        3: AI_Speak(caster, "Mortal, you shall soon be my slave!");
    endcase
    var thecritter  := CreateNpcFromTemplate("skeleton", caster.x, caster.y -2, caster.z, 0);
    var thecritter2 := CreateNpcFromTemplate("zombie", caster.x, caster.y - 1, caster.z, 0);
    var thecritter3 := CreateNpcFromTemplate("skeleton", caster.x, caster.y + 1, caster.z, 0);
    PlaySoundEffect(thecritter,SFX_SPELL_ENERGY_VORTEX);
    SetObjProperty(thecritter, "summoned",1);
    SetObjProperty(thecritter2, "summoned",1);
    SetObjProperty(thecritter3, "summoned",1);
    SetObjProperty(thecritter, "master", caster.serial);
    SetObjProperty(thecritter2, "master", caster.serial);
    SetObjProperty(thecritter3, "master", caster.serial);
    Detach();
    SetMana(caster, Cint(GetMana(caster) - 20));
    sleep( (GetEffectiveSkill( caster, SKILLID_MAGERY ) ) );
    PlaySoundEffect( thecritter, SFX_SPELL_MANA_VAMPIRE );
    PlaySoundEffect( thecritter2, SFX_SPELL_MANA_VAMPIRE );
    PlaySoundEffect( thecritter3, SFX_SPELL_MANA_VAMPIRE );
    PlayStationaryEffect( thecritter.x, thecritter.y, thecritter.z, FX_SMOKE, 0xa, 0xa );
    PlayStationaryEffect( thecritter2.x, thecritter2.y, thecritter2.z, FX_SMOKE, 0xa, 0xa );
    PlayStationaryEffect( thecritter3.x, thecritter3.y, thecritter3.z, FX_SMOKE, 0xa, 0xa );
    MoveCharacterToLocation(thecritter,0,0,0,MOVECHAR_FORCELOCATION);
    MoveCharacterToLocation(thecritter2,0,0,0,MOVECHAR_FORCELOCATION);
    MoveCharacterToLocation(thecritter3,0,0,0,MOVECHAR_FORCELOCATION);
    ApplyRawDamage(thecritter, GetHp(thecritter) + 3);
    ApplyRawDamage(thecritter2, GetHp(thecritter) + 3);
    ApplyRawDamage(thecritter3, GetHp(thecritter) + 3);
  endif
endfunction