/////////////////////////////////////////////////////////////////////////////
//
//	interact.inc
//
//	Author: Caine
//	Date:	09/11/2000 - work in progress
//
//	Description:	Interaction library (events etc.)
//			Current Functions :
//		09/11/2000:	function i_broadcast_event ( evsource, evtype, range := 10, n_times := -1)
//		09/11/2000:	function i_broadcast_event_specific ( evsource, evtype, text, range := 10, n_times := -1)
//
//	Notes:		Requires the os.em (sendevent func)
//			My text.inc (t_in function)
//			I don't have extensive error-checking (none infact) so be sure you send the correct params to the funcs.			
//			
//	Dependencies: 	function i_broadcast_event
//				-> Requires SendEvent 			-> 	uo.em
// 				-> Requires Listmobilesnearlocation	-> 	uo.em
//
//			function i_broadcast_event_specific 
//				-> Requires SendEvent 			-> 	uo.em
//				-> Required Listmobilesnearlocation	->	uo.em
//				-> Requires t_in 			->	text.inc
//			
//	caine 09/11/2000:	Created and used for the first time
//
//	BUGS:		None that i've found just yet ;)
//
//
/////////////////////////////////////////////////////////////////////////////

use uo;

include "text"; // my text.inc

const DEF_RANGE := 10; // default range from source to send event...

/////////////////////////////////////////////////////////////////////
//
//	Common params to the i_broadcast functions:
//		evsource	:	event source
//		evtype		:	event type
//		range		:	event send range
//		n_times		:	how many before stop ( -1 is all mobiles in range )
//
/////////////////////////////////////////////////////////////////////
		
/////////////////////////////////////////////////////////////////////
//
// 	function sends an event to *ALL* surrounding mobiles
//
// NOTE: returns 1 if an event was sent, else returns 0
//
// eg. i_broadcast_event ( me, 16 ) = all mobiles around me get sent event 16
//	OR
//     i_broadcast_event ( me, 16, 2, 3 ) = only 3 mobiles 2 units away will get sent event 16
//
/////////////////////////////////////////////////////////////////////
function i_broadcast_event ( evsource, evtype, range := DEF_RANGE, n_times := -1)
	
	//set_critical(1);
	local ev array;
	local n_count := 0;
	ev.+source := evsource;
	ev.+type := evtype;
	
	local mobiles_near := listmobilesnearlocation( evsource.x, evsource.y, evsource.z, range );
	
	if ( len(mobiles_near) <= 1 )
		//print("no one around to send to"); //debug
		return(0); // there was no one to send the event to, return 0
	endif
	
	// forcing event on every mobile...
	foreach person in mobiles_near 
		if ( person.serial != ev.source.serial ) // exclude source
			//print("Sending event " + ev.type + " from " + ev.source.name + " to " + person.name); //debug
			SendEvent( person, ev); 
				if (n_times != -1)
					n_count := n_count + 1;
					if (n_count >= n_times)
						return(1); // an event was sent to a mobile
					endif
				endif
		endif
	endforeach
	
	return(1); // an event was sent to a mobile
	//set_critical(0);
endfunction

/////////////////////////////////////////////////////////////////////
//
// 	function sends an event to specific surrounding mobiles (by name)
//
// NOTE: returns 1 if an event was sent, else returns 0
//
// Special params: 
//	text	:	text from the targeted mobiles name
//
// eg. i_broadcast_event_specific ( me, 16, "guard" ) = all mobiles around me, with "guard" in the name, get sent event 16
//
/////////////////////////////////////////////////////////////////////
function i_broadcast_event_specific ( evsource, evtype, text, range := DEF_RANGE, n_times := -1)
	
	//set_critical(1);
	var ev array;
	var n_count := 0;
	var ev_sent := 0;
	ev.+source := evsource;
	ev.+type := evtype;
	
	var mobiles_near := listmobilesnearlocation( evsource.x, evsource.y, evsource.z, range );
	
	if ( len(mobiles_near) <= 1 )
		//print("no one around to send to"); //debug
		return(0); // there was no one to send the event to, return 0
	endif
	
	// forcing event on specific mobiles... (using mobile.name)
	foreach person in mobiles_near 
		if ( person.serial != ev.source.serial ) // exclude source
			if ( t_in(text, person.name) )
				//print("Sending event " + ev.type + " from " + ev.source.name + " to " + person.name); //debug
				SendEvent( person, ev ); 
				ev_sent := 1; // sent at least ONE event to a mobile		
					if (n_times != -1)
						n_count := n_count + 1;
						if (n_count >= n_times)
							return(1); // an event was sent to a mobile
						endif
					endif
			endif
		endif
	endforeach
	
	if (ev_sent) // an event was sent
		return(1); // an event was sent to a mobile
	else	// no event sent because specific mobile was not in range
		//print("mobile in range, but not the specified mobile"); //debug
		return(0); // no event sent
	endif
	//set_critical(0);
endfunction
