use util;
use uo;
use os;
use basic;
use datafile;



const AUTOACCOUNT_DATAFILENAME := ":AAC:AutoAccount_Reference";

const AUTOACCOUNT_ACCOUNTLENGTH_MIN := 4;
const AUTOACCOUNT_ACCOUNTLENGTH_MAX := 12;

const AUTOACCOUNT_REF_ACCOUNTNAME := "AccountName";
const AUTOACCOUNT_REF_PASSWORD := "GeneratedPassword";
const AUTOACCOUNT_REF_IP := "Ip";
const AUTOACCOUNT_ACCOUNT_EMAIL := "RegisteredEmail";
const AUTOACCOUNT_ACCOUNT_KEY := "ActivateKey";
const AUTOACCOUNT_ACCOUNT_CREATED := "CreatedAT";



//CheckDataFile()
//
//Checks if the reference file is present, if not, create it..
//returns a reference to the reference file
function AutoAccount_CheckDataFile()
	var AutoAccountReference := OpenDataFile(AUTOACCOUNT_DATAFILENAME);
	if (!AutoAccountReference)
	   AutoAccountReference := CreateDataFile(AUTOACCOUNT_DATAFILENAME, DF_KEYTYPE_STRING);
	endif
	return AutoAccountReference;
endfunction





//CheckAccountName(string AccountName)
//
//Checks if AccountName isn't already in use, if it's the right length etc.
//returns 1 if correct, 0 if not
function AutoAccount_CheckAccountName(AccountName)
	var AutoAccount_Account := FindAccount(AccountName);
	if(AutoAccount_Account)
		return 0;
	elseif(len(AccountName) < AUTOACCOUNT_ACCOUNTLENGTH_MIN)
		return 0;
	elseif(len(AccountName) > AUTOACCOUNT_ACCOUNTLENGTH_MAX)
		return 0;
	endif
	return 1;
endfunction


//Email_Exists(string Email)
//
//Checks if the e-mail address Email hasn't been used yet
//returns 1 if correct, 0 if not
function AutoAccount_CheckEmail(Email)
	var AutoAccountReference := AutoAccount_CheckDataFile();
	var Element := AutoAccountReference.FindElement(lower(Email));
	if(Element)
		var Account := Element.GetProp(AUTOACCOUNT_REF_ACCOUNTNAME);
		if (Account)
			return 0;
		else
			return 1;
		endif
	endif
	return 1;
endfunction

//CreateAccount(string Account, string Password, string Email, string Ip)
//
//Double checks entered info (Account/Email), but should be checked beforehand
//if you want to relay info back to the user
//if everything is correct it will create the account and set the info to the
//account reference file
function AutoAccount_CreateAccount(Account, Password, Email,Key,  Ip)
	if(AutoAccount_CheckAccountName(Account) == 0)
		return 0;
	elseif(AutoAccount_CheckEmail(Email) == 0)
		return 0;
	endif
	var AutoAccountReference := AutoAccount_CheckDataFile();

	var AccountElement := AutoAccountReference.FindElement(lower(Email));
	if(!AccountElement)
		AccountElement := AutoAccountReference.CreateElement(lower(Email));
	endif
	if(!AccountElement)
		return 0;
	endif
	var AccountObj := CreateAccount(Account, Password, 0);
	if(!AccountObj)
		AccountElement.DeleteElement(lower(Email));
		return 0;
	endif
	AccountElement.SetProp(AUTOACCOUNT_REF_ACCOUNTNAME, lower(Account));
	AccountElement.SetProp(AUTOACCOUNT_REF_PASSWORD, lower(Password));
	AccountElement.SetProp(AUTOACCOUNT_REF_IP, Ip);
	AccountObj.SetProp(AUTOACCOUNT_ACCOUNT_EMAIL, lower(Email));
	AccountObj.SetProp(AUTOACCOUNT_ACCOUNT_KEY, lower(Key));
	AccountObj.SetProp(AUTOACCOUNT_ACCOUNT_CREATED, ReadGameClock());
	return 1;
endfunction

function AutoAccount_NotActivated(Account, nogetdate := 0)
	//print("Finding " + Account);
	var AutoAccount_Account := FindAccount(Account);
	if(!AutoAccount_Account)
		return 0;
	endif
	var prop := AutoAccount_Account.GetProp(AUTOACCOUNT_ACCOUNT_KEY);
	//print("ACCT KEY Prop:(" + AUTOACCOUNT_ACCOUNT_KEY + ") " + prop);
	if(prop != error)
		if(nogetdate == 0)
			return AutoAccount_GetCreationDate(Account);
		else
			return 1;
		endif
	endif
	return 0;
endfunction
function AutoAccount_GetCreationDate(Account)
	if(AutoAccount_NotActivated(Account, 1) == 0)
		return ReadGameClock();
	endif
	var AutoAccount_Account := FindAccount(Account);

	return AutoAccount_Account.GetProp(AUTOACCOUNT_ACCOUNT_CREATED);
endfunction

function AutoAccount_CheckKey(Account, Key)
	if(AutoAccount_NotActivated(Account) == 0)
		return 0;
	endif
	var AutoAccount_Account := FindAccount(Account);
	var prop := AutoAccount_Account.GetProp(AUTOACCOUNT_ACCOUNT_KEY);
	if(prop == Key)
		return 1;
	endif
	return 0;
endfunction
function AutoAccount_CheckAccountMail(Account, Email)
	var TheMail := AutoAccount_GetAccountEmail(Account);
	if(TheMail == 0)
		return 0;
	endif
	if(lower(TheMail) == lower(Email))
		return 1;
	else
		return 0;
	endif
endfunction

function AutoAccount_ActivateAccount(Account)
	var AutoAccount_Account := FindAccount(Account);
	if(!AutoAccount_Account)
		return 0;
	endif
	if(AutoAccount_NotActivated(Account) == 0)
		return 0;
	endif
	AutoAccount_Account.EraseProp(AUTOACCOUNT_ACCOUNT_KEY);
	AutoAccount_Account.Enable();
	return 1;
endfunction


//GetAccountEmail(string Account)
//
//Gets the email address from Account
//returns 0 if the account does not exist or does not have an email assigned
function AutoAccount_GetAccountEmail(Account)
	var AccountInfo := FindAccount(Account);
	if(!AccountInfo)
		return 0;
	endif
	var Email := AccountInfo.GetProp(AUTOACCOUNT_ACCOUNT_EMAIL);
	if(!Email)
		return 0;
	endif
	return Email;
endfunction

function AutoAccount_SetRawAccountInfo(Account, Email)
	print("Adding raw :: " + Account + " - " + Email);
	var AccountInfo := FindAccount(Account);
	if(!AccountInfo)
		return 0;
	endif
	AccountInfo.SetProp(AUTOACCOUNT_ACCOUNT_EMAIL, Email);
	var AutoAccountReference := AutoAccount_CheckDataFile();
	var Element := AutoAccountReference.FindElement(lower(Email));
	if(!Element)
		Element := AutoAccountReference.CreateElement(lower(Email));
	endif
	Element.SetProp(AUTOACCOUNT_REF_ACCOUNTNAME, lower(Account));
	Element.SetProp(AUTOACCOUNT_REF_PASSWORD, ""+ReadGameClock());
	Element.SetProp(AUTOACCOUNT_REF_IP, "");
endfunction


function AutoAccount_RemoveEmail(Email)
	var AutoAccountReference := AutoAccount_CheckDataFile();
	var Element := AutoAccountReference.FindElement(lower(Email));
	if(!Element)
		return;
	else
		AutoAccountReference.DeleteElement(lower(Email));
		return;
	endif
endfunction