/*
 * SaveGump.src
 *
 * Shows a gump that indicates the world is saving. The gump will close once
 * the world save is over.
 *
 * @author	shaDoze (shadoze@neverlands.org)
 * @date	August 25, 2004
 */

use uo;
use os;

include "../pkg/foundations/gumps/gumps";

const SG_GUMP_PID	:= "SaveGumpPID";

/**
 * Display's the "saving world state" gump to a player.
 *
 * @param	who	Player object to create the gump for.
 */
function CreateSaveGump( who )

	// I do not recommend this, as it could block the player's view without
	// the possibility to remove the gump.

	// GFNoClose();
	// GFNoDispose();
	// GFNoMove();

	GFInitGump( 200, 200 );
	GFResizePic( 0, 0, 9270, 395, 120 );

	GFTilePic( 328, 41, 4168 );

	GFTextLine( 118, 20, 0x480, "Saving World State" );
	GFTextLine( 70, 55, 255, "Please be patient while the world is" );
	GFTextLine( 70, 75, 255, "saved. This can take a few seconds." );

	GFTilePic( 25, 40, 4171 );

	GFSendGump( who );

endfunction

/**
 * Shows a save gump to all players online.
 */
function ShowSaveGump()

	SetGlobalProperty( SG_GUMP_PID, GetPid() );

	foreach player in EnumerateOnlineCharacters()
		CreateSaveGump( player );
	endforeach

endfunction

/**
 * Shows a save gump to all players online.
 */
function CloseSaveGump()

	foreach player in EnumerateOnlineCharacters()
		DestroySaveGump( player );
	endforeach

	EraseGlobalProperty( SG_GUMP_PID );

endfunction

/**
 * Removes the "saving world state" gump for a player.
 *
 * @param	who	Player object to remove the gump for.
 */
function DestroySaveGump( who ) 

	var pid := CInt( GetGlobalProperty( SG_GUMP_PID ) );

	if( pid > 0 )
		var returnValue := 0;

		var packetString := "BF000D0004"; 
		packetString := packetString + fixPacketLength( Hex( pid ), 4 ); 
		packetString := packetString + fixPacketLength( Hex( returnValue ), 4 ); 
		SendPacket( who, packetString );
	endif

endfunction 

/**
 * Creates a valid packet string, consisting  of a number of bytes.
 *
 * @param	packetString	The string to send.
 * @param	byteLength	The number of bytes the packet needs to be.
 *				One byte is counted as two characters.
 *
 * @return	Valid packet string.
 */
function fixPacketLength( packetString, byteLength ) 

	if( !packetString || packetString == error ) 
		packetString := 0; 
	endif 

	packetString := CStr(packetString); 

	if( packetString[1,2] == "0x" ) 
		packetString := cStr( packetString[3, len( packetString )] ); 
	endif 

	if( len( packetString) > byteLength * 2 ) 
		var extraBytes := ( byteLength * 2 ) - len( packetString ); 
		return cStr(packetString[ extraBytes,len( packetString )] ); 
	endif 

	while( len(packetString) < byteLength * 2 ) 
		packetString := "0" + packetString; 
	endwhile 

	return packetString; 

endfunction