include "include/attributes";

const DEFAULT_RATING := 1200;
const DEFAULT_UNCERTAINTY := 50;
const MINIMUM_UNCERTAINTY := 10;
const UNCERTAINTY_INCREASE_TIME := 60*60*24;

function getPvpRanking(char);
  var ranking := GetObjProperty(char, "PVPRanking"); // get prop array;
  if(!ranking) // not yet made
    ranking := struct{};
    ranking .+ rating;
    ranking .+ uncertainty;
    ranking .+ lastpvp;
    ranking .+ pvpkills;
    ranking .+ pvpdeaths;

    ranking.rating      := DEFAULT_RATING;
    ranking.uncertainty := DEFAULT_UNCERTAINTY;
    ranking.lastpvp     := ReadGameClock();
    ranking.pvpkills    := 0;
    ranking.pvpdeaths   := 0;
  endif

  var now := ReadGameClock();
  if(now - ranking.lastpvp > UNCERTAINTY_INCREASE_TIME)
    ranking.uncertainty := ranking.uncertainty + Floor( (now-ranking.lastpvp) / UNCERTAINTY_INCREASE_TIME); // give 1 more uncertainty for each UNCERTAINTY_INCREASE_TIME that char hasn't been killed or died.
    if(ranking.uncertainty > DEFAULT_UNCERTAINTY)
      ranking.uncertainty := DEFAULT_UNCERTAINTY;
    endif
    ranking.lastpvp := ReadGameClock();
  endif
  
  return ranking;
endfunction

function setPvpRanking(char, ranking)
  SetObjProperty(char, "PVPRanking", ranking);
endfunction

function newRating(char, opponent, won);

  var diff := cdbl(opponent.rating - char.rating);
  var rd := cdbl(diff / cdbl(400));
  var pw := Pow(10, rd);

  var cw := cdbl(1) / (cdbl(1) + pw);

  var scoreBonus := cdbl(32) * (won - cw);
  print(scoreBonus);
  char.rating := char.rating + scoreBonus;

  if(won == 1)
    char.pvpkills := char.pvpkills + 1;
  else
    char.pvpdeaths := char.pvpdeaths + 1;
  endif
  
  char.lastpvp := ReadGameClock();

  return char;
endfunction

function sortRankingList(list)

var x;
var z;
var temp := {};
var temp1 := {};

for(x := list.size(); x > 0; x := x - 1)
	sleepms(1);
	for(z := x; z < list.size(); z := z + 1)
		if(list[z][2] < list[z+1][2])
			temp := list[z];
			list[z] := list[z+1];
			list[z+1] := temp;
		endif
	endfor
endfor
list.shrink(50);
return list;
endfunction

function addGuildPoints(list, guild, points)

var i;
var found := 0;
for(i := 0; i < list.size(); i := i + 1)
	if(list[i][1] == guild)
		list[i][2] := list[i][2] + points;
		found := 1;
	endif
endfor
if(!found)
 list.append({guild, points});
endif
return list;
endfunction

function sortGuildList(list)

var x;
var z;
var temp := {};
var temp1 := {};

for(x := list.size(); x > 0; x := x - 1)
	for(z := x; z < list.size(); z := z + 1)
		if(list[z][2] < list[z+1][2])
			temp := list[z];
			list[z] := list[z+1];
			list[z+1] := temp;
		endif
		sleepms(1);
	endfor
endfor
list.shrink(10);
return list;
endfunction